/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.cat.glossary.core;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

import benten.cat.glossary.core.data.AbstractGlossaryDataPlugin;
import blanco.omegat.util.glossary.MyGlossaryManager;

/**
 * The activator class controls the plug-in life cycle
 */
public class CatGlossaryCorePlugin extends AbstractUIPlugin {

	// The plug-in ID
	public static final String PLUGIN_ID = CatGlossaryCorePlugin.class.getPackage().getName();

	// The shared instance
	private static CatGlossaryCorePlugin plugin;

	private static final String DEFAULT_GLOSSARY = "Default Glossary";
	private static MyGlossaryManager gm;
	private final Map<String, AbstractGlossaryDataPlugin> glossaryMap = new HashMap<String, AbstractGlossaryDataPlugin>();

	/**
	 * The constructor
	 */
	public CatGlossaryCorePlugin() {
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * org.eclipse.ui.plugin.AbstractUIPlugin#start(org.osgi.framework.BundleContext
	 * )
	 */
	@Override
	public void start(BundleContext context) throws Exception {
		super.start(context);
		plugin = this;
		gm = new MyGlossaryManager();
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * org.eclipse.ui.plugin.AbstractUIPlugin#stop(org.osgi.framework.BundleContext
	 * )
	 */
	@Override
	public void stop(BundleContext context) throws Exception {
		plugin = null;
		super.stop(context);
	}

	/**
	 * Returns the shared instance
	 *
	 * @return the shared instance
	 */
	public static CatGlossaryCorePlugin getDefault() {
		return plugin;
	}

	/**
	 * Returns an image descriptor for the image file at the given plug-in
	 * relative path
	 *
	 * @param path
	 *            the path
	 * @return the image descriptor
	 */
	public static ImageDescriptor getImageDescriptor(String path) {
		return imageDescriptorFromPlugin(PLUGIN_ID, path);
	}

	/**
	 * Logs the given message.
	 *
	 * @param message
	 *            the message to log
	 */
	public static void log(String message) {
		plugin.getLog().log(new Status(IStatus.WARNING, PLUGIN_ID, message));
	}

	/**
	 * Logs the given exception.
	 *
	 * @param ex
	 *            the exception to log
	 */
	public static void log(final Throwable ex) {
		plugin.getLog().log(new Status(IStatus.WARNING, PLUGIN_ID, -1, ex.toString(), ex));
	}

	@SuppressWarnings("static-access")
	public void loadGlossary() {

		IExtensionRegistry registry = Platform.getExtensionRegistry();
		IExtensionPoint point = registry.getExtensionPoint(PLUGIN_ID + ".data");
		System.out.println(point);
		if (point == null)
			return;

		IExtension[] extensions = point.getExtensions();
		for (int i = 0; i < extensions.length; i++) {
			IConfigurationElement[] cfgElems = extensions[i].getConfigurationElements();
			for (int j = 0; j < cfgElems.length; j++) {
				IConfigurationElement cfgElem = cfgElems[j];
				try {
					String name = cfgElem.getAttribute("name");
					Object obj = cfgElem.createExecutableExtension("class");
					if (obj instanceof AbstractGlossaryDataPlugin) {
						glossaryMap.put(name, (AbstractGlossaryDataPlugin) obj);
					}
				} catch (CoreException e) {
					CatGlossaryCorePlugin.log(e);
				}
			}
		}

		AbstractGlossaryDataPlugin glossaryDatalugin = glossaryMap.get(DEFAULT_GLOSSARY);
		if (glossaryDatalugin == null)
			return;

		try {
			// FIXME ファイルの読み込み方
			URL entry = glossaryDatalugin.getDefault().getBundle().getEntry("/glossary");
			String pluginDirectory = FileLocator.resolve(entry).getPath();
			File file = new File(pluginDirectory);
			gm.loadGlossaryFiles(file);
		} catch (IOException e) {
			CatGlossaryCorePlugin.log(e);
		}
	}

	public void unloadGlossary() {
		gm.clear();
	}

	public MyGlossaryManager getGm() {
		return gm;
	}
}
