/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.cat.glossary.ui;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

import benten.cat.glossary.core.BentenGlossaryEngine;
import benten.ui.UiPlugin;
import benten.ui.UiPlugin.Key;

/**
 * The activator class controls the plug-in life cycle
 */
public class CatGlossaryUiPlugin extends AbstractUIPlugin {

	// The plug-in ID
	public static final String PLUGIN_ID = CatGlossaryUiPlugin.class.getPackage().getName();

	// The shared instance
	private static CatGlossaryUiPlugin plugin;

	private BentenGlossaryEngine glossaryEngine;

	private final Map<String, BentenGlossaryEngine> glossaryMap = new HashMap<String, BentenGlossaryEngine>();

	/**
	 * The constructor
	 */
	public CatGlossaryUiPlugin() {
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * org.eclipse.ui.plugin.AbstractUIPlugin#start(org.osgi.framework.BundleContext
	 * )
	 */
	@Override
	public void start(BundleContext context) throws Exception {
		super.start(context);
		plugin = this;
		loadExtensions();

		try {
			// TODO クラス名を外部化。
			Class.forName("benten.cat.glossary.core.omegat.DefaultGlossaryDriver");
		} catch (ClassNotFoundException e) {
			log("デフォルトのグロッサリー・エンジンの読み込みに失敗しました。");
			log(e);
		}
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * org.eclipse.ui.plugin.AbstractUIPlugin#stop(org.osgi.framework.BundleContext
	 * )
	 */
	@Override
	public void stop(BundleContext context) throws Exception {
		plugin = null;
		super.stop(context);
	}

	/**
	 * Returns the shared instance
	 *
	 * @return the shared instance
	 */
	public static CatGlossaryUiPlugin getDefault() {
		return plugin;
	}

	/**
	 * Returns an image descriptor for the image file at the given plug-in
	 * relative path
	 *
	 * @param path
	 *            the path
	 * @return the image descriptor
	 */
	public static ImageDescriptor getImageDescriptor(String path) {
		return imageDescriptorFromPlugin(PLUGIN_ID, path);
	}

	/**
	 * Logs the given message.
	 *
	 * @param message
	 *            the message to log
	 */
	public static void log(String message) {
		plugin.getLog().log(new Status(IStatus.WARNING, PLUGIN_ID, message));
	}

	/**
	 * Logs the given exception.
	 *
	 * @param ex
	 *            the exception to log
	 */
	public static void log(final Throwable ex) {
		plugin.getLog().log(new Status(IStatus.WARNING, PLUGIN_ID, -1, ex.toString(), ex));
	}

	public void loadExtensions() {

		IExtensionRegistry registry = Platform.getExtensionRegistry();
		IExtensionPoint point = registry.getExtensionPoint(PLUGIN_ID + ".engine");
		System.out.println(point);
		if (point == null)
			return;

		IExtension[] extensions = point.getExtensions();
		for (int i = 0; i < extensions.length; i++) {
			IConfigurationElement[] cfgElems = extensions[i].getConfigurationElements();
			for (int j = 0; j < cfgElems.length; j++) {
				IConfigurationElement cfgElem = cfgElems[j];
				try {
					String name = cfgElem.getAttribute("name");
					Object obj = cfgElem.createExecutableExtension("class");
					if (obj instanceof BentenGlossaryEngine) {
						glossaryMap.put(name, (BentenGlossaryEngine) obj);
					}
				} catch (CoreException e) {
					log(e);
				}
			}

		}

		IPreferenceStore store = UiPlugin.getDefault().getPreferenceStore();
		String tmName = store.getString(Key.TM_NAME.name());
		BentenGlossaryEngine glossaryEngine = glossaryMap.get(tmName);
		if (glossaryEngine != null) {
			glossaryEngine.load();
			this.glossaryEngine = glossaryEngine;
		}

	}

	// private void loadDictionary() {
	// try {
	// URL entry = GlossaryPlugin.getDefault().getBundle().getEntry(
	// "/file_properties.xml");
	// if (entry == null){
	// return;
	// }
	//
	// GlossaryPlugin.log(FileLocator.resolve(entry).getPath());
	// JWNL.initialize(new FileInputStream(FileLocator.resolve(entry)
	// .getPath()));
	// dic = Dictionary.getInstance();
	// } catch (FileNotFoundException e) {
	// // XTODO Auto-generated catch block
	// e.printStackTrace();
	// } catch (JWNLException e) {
	// // XTODO Auto-generated catch block
	// e.printStackTrace();
	// } catch (IOException e) {
	// // XTODO Auto-generated catch block
	// e.printStackTrace();
	// }
	// }

	public BentenGlossaryEngine getGlossaryEngine() {
		return glossaryEngine;
	}

	// public List<String> getIndexWordList(String word) {
	// ArrayList<String> result = new ArrayList<String>();
	//
	// if (word == null) {
	// return result;
	// }
	//
	// if (dic == null){
	// return result;
	// }
	//
	// MorphologicalProcessor morphProc = dic.getMorphologicalProcessor();
	// HashSet<String> resultHashSet = new HashSet<String>();
	// try {
	// List<POS> allPos = POS.getAllPOS();
	// for (POS pos : allPos) {
	// List<String> baseForms = morphProc.lookupAllBaseForms(pos, word);
	// for (String baseForm: baseForms) {
	// if (!word.equals(baseForm)) {
	// resultHashSet.add(baseForm);
	// System.out.println("[word] " + baseForm);
	// System.out.println(pos);
	// }
	//
	// }
	// }
	// } catch (JWNLException e) {
	// // XTODO Auto-generated catch block
	// e.printStackTrace();
	// }
	//
	// result.addAll(resultHashSet);
	// return result;
	// }
	//
}
