/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.cat.translation.ui;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

import benten.cat.translation.core.BentenTmEngine;
import benten.cat.translation.core.BentenTmSearchResult;
import benten.ui.UiPlugin;
import benten.ui.UiPlugin.Key;

/**
 * The activator class controls the plug-in life cycle
 */
public class CatTranslationUiPlugin extends AbstractUIPlugin {

	// The plug-in ID
	public static final String PLUGIN_ID = CatTranslationUiPlugin.class.getPackage().getName();

	// The shared instance
	private static CatTranslationUiPlugin plugin;

	Map<String, BentenTmEngine> tmMap = new HashMap<String, BentenTmEngine>();
	BentenTmEngine tm = null;

	/**
	 * The constructor
	 */
	public CatTranslationUiPlugin() {
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * org.eclipse.ui.plugin.AbstractUIPlugin#start(org.osgi.framework.BundleContext
	 * )
	 */
	@Override
	public void start(BundleContext context) throws Exception {
		super.start(context);
		plugin = this;
		loadExtensions();
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * org.eclipse.ui.plugin.AbstractUIPlugin#stop(org.osgi.framework.BundleContext
	 * )
	 */
	@Override
	public void stop(BundleContext context) throws Exception {
		plugin = null;
		super.stop(context);
	}

	/**
	 * Returns the shared instance
	 *
	 * @return the shared instance
	 */
	public static CatTranslationUiPlugin getDefault() {
		return plugin;
	}

	/**
	 * Returns an image descriptor for the image file at the given plug-in
	 * relative path
	 *
	 * @param path
	 *            the path
	 * @return the image descriptor
	 */
	public static ImageDescriptor getImageDescriptor(String path) {
		return imageDescriptorFromPlugin(PLUGIN_ID, path);
	}

	/**
	 * Logs the given message.
	 *
	 * @param message
	 *            the message to log
	 */
	public static void log(String message) {
		plugin.getLog().log(new Status(IStatus.WARNING, PLUGIN_ID, message));
	}

	/**
	 * Logs the given exception.
	 *
	 * @param ex
	 *            the exception to log
	 */
	public static void log(final Throwable ex) {
		plugin.getLog().log(new Status(IStatus.WARNING, PLUGIN_ID, -1, ex.toString(), ex));
	}

	public Map<String, BentenTmEngine> getTmMap() {
		return tmMap;
	}

	public void loadExtensions() {
		IExtensionRegistry registry = Platform.getExtensionRegistry();
		IExtensionPoint point = registry.getExtensionPoint(PLUGIN_ID + ".engine");
		if (point == null)
			return;

		// このシステムに登録されている、すべての翻訳メモリー・エンジンの一覧を取得。
		// 取得された翻訳メモリー･エンジンの名前を記憶。
		final IExtension[] extensions = point.getExtensions();
		for (int i = 0; i < extensions.length; i++) {
			IConfigurationElement[] cfgElems = extensions[i].getConfigurationElements();
			for (int j = 0; j < cfgElems.length; j++) {
				IConfigurationElement cfgElem = cfgElems[j];
				try {
					String name = cfgElem.getAttribute("name");
					Object helloObj;
					helloObj = cfgElem.createExecutableExtension("class");
					if (helloObj instanceof BentenTmEngine) {
						tmMap.put(name, (BentenTmEngine) helloObj);
					}
				} catch (CoreException e) {
					log(e);
				}
			}
		}

		// 「設定」から翻訳メモリー･エンジンの名称を取得。
		// 該当する翻訳メモリー・エンジンの翻訳メモリーをロードする。
		IPreferenceStore store = UiPlugin.getDefault().getPreferenceStore();
		String tmName = store.getString(Key.TM_NAME.name());
		BentenTmEngine tm = tmMap.get(tmName);
		if (tm != null) {
			tm.load();
			this.tm = tm;
		}
	}

	public List<BentenTmSearchResult> fuzzyMatch(String string) {
		synchronized (this) {
			if (tm == null) {
				log("fuzzyMatch:TMエンジンがロードされていません。");
				return new ArrayList<BentenTmSearchResult>();
			}
			IPreferenceStore store = UiPlugin.getDefault().getPreferenceStore();
			boolean ignoreWhitespace = store.getBoolean(Key.IGNORE_WHITESPACE_TM_REFERENCE.name());
			if (ignoreWhitespace) {
				string = string.trim().replaceAll("\\s+", " ");
			}
			return tm.fuzzySearch(string);
		}
	}
}
