/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.cat.translation.ui;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

import benten.ui.UiPlugin;
import benten.ui.UiToolkit;
import benten.ui.UiPlugin.Key;

/**
 * TM 設定ページです。
 */
public class CatTranslationUiPreferencePage extends PreferencePage implements IWorkbenchPreferencePage {

	private final UiToolkit toolkit = new UiToolkit();
	private Text tmText;
	private Button ignoreWhitespaceTmReferenceCheck;
	private Button ignoreWhitespaceTmxExtractCheck;

	public CatTranslationUiPreferencePage() {
	}

	public CatTranslationUiPreferencePage(String title) {
		super(title);
	}

	public CatTranslationUiPreferencePage(String title, ImageDescriptor image) {
		super(title, image);
	}

	@Override
	protected Control createContents(Composite parent) {
		Composite composite = new Composite(parent, SWT.NONE);
		composite.setLayout(toolkit.createGridLayout(1, 0));

		Composite c = new Composite(composite, SWT.NONE);
		c.setLayout(toolkit.createGridLayout(2, 0));
		c.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		toolkit.createLabel(c, "翻訳メモリー:");

		tmText = new Text(c, SWT.SINGLE | SWT.BORDER);
		tmText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		IPreferenceStore sotre = UiPlugin.getDefault().getPreferenceStore();
		tmText.setText(sotre.getString(Key.TM_NAME.name()));

		toolkit.createLabel(composite, "");

		Group g = new Group(composite, SWT.BOLD);
		g.setLayout(toolkit.createGridLayout(1, 5));
		g.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		g.setText("空白や改行の扱い");
		IPreferenceStore store = UiPlugin.getDefault().getPreferenceStore();

		ignoreWhitespaceTmReferenceCheck = new Button(g, SWT.CHECK);
		ignoreWhitespaceTmReferenceCheck.setText("TM 参照時に空白や改行を無視(&R)");
		ignoreWhitespaceTmReferenceCheck.setSelection(store.getBoolean(Key.IGNORE_WHITESPACE_TM_REFERENCE.name()));

		ignoreWhitespaceTmxExtractCheck = new Button(g, SWT.CHECK);
		ignoreWhitespaceTmxExtractCheck.setText("TMX 抽出時に空白や改行を無視(&E)");
		ignoreWhitespaceTmxExtractCheck.setSelection(store.getBoolean(Key.IGNORE_WHITESPACE_TMX_EXTRACT.name()));

		return composite;
	}

	public void init(IWorkbench workbench) {
	}

	@Override
	public boolean performOk() {
		IPreferenceStore store = UiPlugin.getDefault().getPreferenceStore();
		store.setValue(Key.TM_NAME.name(), tmText.getText());
		store.setValue(Key.IGNORE_WHITESPACE_TM_REFERENCE.name(), ignoreWhitespaceTmReferenceCheck.getSelection());
		store.setValue(Key.IGNORE_WHITESPACE_TMX_EXTRACT.name(), ignoreWhitespaceTmxExtractCheck.getSelection());
		return true;
	}

	@Override
	protected void performDefaults() {
		IPreferenceStore store = UiPlugin.getDefault().getPreferenceStore();
		tmText.setText(store.getDefaultString(Key.TM_NAME.name()));
		ignoreWhitespaceTmReferenceCheck.setSelection(store
				.getDefaultBoolean(Key.IGNORE_WHITESPACE_TM_REFERENCE.name()));
		ignoreWhitespaceTmxExtractCheck.setSelection(store.getDefaultBoolean(Key.IGNORE_WHITESPACE_TMX_EXTRACT.name()));
	}
}
