/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.cat.ui;

import java.util.LinkedList;
import java.util.List;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.preferences.IEclipsePreferences.IPreferenceChangeListener;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

/**
 * エディター UI プラグインです。
 */
public class CatUiPlugin extends AbstractUIPlugin {

	public static final String PLUGIN_ID = CatUiPlugin.class.getPackage().getName();
	private static CatUiPlugin plugin;
	private IEditorPart activeEditorPart;
	private final List<IPreferenceChangeListener> preferenceChangeListners = new LinkedList<IPreferenceChangeListener>();

	public static enum Key {
		SHOW_LINE_BREAK
	}

	public CatUiPlugin() {
	}

	@Override
	public void start(BundleContext context) throws Exception {
		super.start(context);
		plugin = this;
	}

	@Override
	public void stop(BundleContext context) throws Exception {
		plugin = null;
		super.stop(context);
	}

	public static CatUiPlugin getDefault() {
		return plugin;
	}

	/**
	 * Logs the given message.
	 *
	 * @param message the message to log
	 */
	public static void log(String message) {
		plugin.getLog().log(new Status(IStatus.WARNING, PLUGIN_ID, message));
		System.out.println(message); //debug
	}

	/**
	 * Logs the given exception.
	 *
	 * @param ex the exception to log
	 */
	public static void log(final Throwable ex) {
		plugin.getLog().log(new Status(IStatus.WARNING, PLUGIN_ID, -1, ex.toString(), ex));
	}

	public Image getImage(String path) {
		Image image = getImageRegistry().get(path);
		if (image == null) {
			ImageDescriptor descriptor = AbstractUIPlugin.imageDescriptorFromPlugin(PLUGIN_ID, path);
			if (descriptor != null) {
				getImageRegistry().put(path, image = descriptor.createImage());
			}
		}
		return image;
	}

	public IEditorPart getActiveEditorPart() {
		return activeEditorPart;
	}

	public void setActiveEditorPart(IEditorPart activeEditorPart) {
		this.activeEditorPart = activeEditorPart;
	}

	public void firePreferenceChange() {
		for (IPreferenceChangeListener listener : preferenceChangeListners) {
			listener.preferenceChange(null);
		}
	}

	public void addPreferenceChangeListner(IPreferenceChangeListener listner) {
		preferenceChangeListners.add(listner);
	}

	public void removePreferenceChangeListner(IPreferenceChangeListener listner) {
		preferenceChangeListners.remove(listner);
	}

	/**
	 * 可視状態のビューを取得します。
	 * @param <T> IViewPart を継承したオブジェクト
	 * @param viewPartClass IViewPart クラス
	 * @return ビュー。取得できない場合は null。
	 */
	@SuppressWarnings("unchecked")
	public static <T extends IViewPart> T getView(Class<T> viewPartClass) {
		IWorkbench workbench = PlatformUI.getWorkbench();
		IWorkbenchWindow window = workbench.getWorkbenchWindows()[0];
		IWorkbenchPage page = window.getActivePage();
		IViewReference[] references = page.getViewReferences();

		for (IViewReference reference : references) {
			IViewPart view = reference.getView(false);
			if (view != null && view.getClass() == viewPartClass) {
				return (T) view;
			}
		}
		return null;
	}
}
