/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.cat.validation;

import java.io.InputStream;

import jp.sourceforge.mergedoc.pleiades.resource.PropertySet;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Plugin;
import org.eclipse.core.runtime.Status;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.osgi.framework.BundleContext;

/**
 * The activator class controls the plug-in life cycle
 */
public class CatValidationPlugin extends Plugin {

	// The plug-in ID
	public static final String PLUGIN_ID = CatValidationPlugin.class.getPackage().getName();

	// The shared instance
	private static CatValidationPlugin plugin;

	/** 用語検証プロパティー */
	public static PropertySet vTermProp = new PropertySet();

	/** 対訳正規表現セット */
	public static PropertySet vTranslationProp = new PropertySet();

	/**
	 * The constructor
	 */
	public CatValidationPlugin() {
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.core.runtime.Plugins#start(org.osgi.framework.BundleContext)
	 */
	@Override
	public void start(BundleContext context) throws Exception {
		super.start(context);
		plugin = this;

		// 用語検証プロパティーのロード
		vTermProp.load(getInputStream("validation-term.properties"));

		// 対訳正規表現セットのロード
		vTranslationProp.load(getInputStream("/validation-translation.properties"));
	}

	private InputStream getInputStream(String fileName) {
		return getClass().getClassLoader().getResourceAsStream("conf/props/" + fileName);
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.core.runtime.Plugin#stop(org.osgi.framework.BundleContext)
	 */
	@Override
	public void stop(BundleContext context) throws Exception {
		plugin = null;
		super.stop(context);
	}

	/**
	 * Returns the shared instance
	 *
	 * @return the shared instance
	 */
	public static CatValidationPlugin getDefault() {
		return plugin;
	}

	/**
	 * Logs the given message.
	 *
	 * @param message the message to log
	 */
	public static void log(String message) {
		plugin.getLog().log(new Status(IStatus.WARNING, PLUGIN_ID, message));
		System.out.println(message); //debug
	}

	/**
	 * Logs the given exception.
	 *
	 * @param ex the exception to log
	 */
	public static void log(final Throwable ex) {
		plugin.getLog().log(new Status(IStatus.WARNING, PLUGIN_ID, -1, ex.toString(), ex));
	}

	/**
	 * 可視状態のビューを取得します。
	 * @param <T> IViewPart を継承したオブジェクト
	 * @param viewPartClass IViewPart クラス
	 * @return ビュー。取得できない場合は null。
	 */
	@SuppressWarnings("unchecked")
	public static <T extends IViewPart> T getView(Class<T> viewPartClass) {
		IWorkbench workbench = PlatformUI.getWorkbench();
		IWorkbenchWindow window = workbench.getWorkbenchWindows()[0];
		IWorkbenchPage page = window.getActivePage();
		IViewReference[] references = page.getViewReferences();

		for (IViewReference reference : references) {
			IViewPart view = reference.getView(false);
			if (view != null && view.getClass() == viewPartClass) {
				return (T) view;
			}
		}
		return null;
	}
}
