/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.cat.validation.validator;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.wst.validation.internal.core.Message;
import org.eclipse.wst.validation.internal.operations.TaskListHelper;
import org.eclipse.wst.validation.internal.provisional.core.IMessage;
import org.eclipse.wst.validation.internal.provisional.core.IReporter;
import org.eclipse.wst.validation.internal.provisional.core.IValidator;
import org.eclipse.wst.xml.core.internal.provisional.document.IDOMModel;

import benten.cat.validation.CatValidationPlugin;

/**
 * 検証メッセージを追加するための翻訳リポーターです。
 */
@SuppressWarnings( { "restriction", "deprecation" })
public class TranslationValidationReporter {

	private final IValidator validator;
	private final IReporter reporter;
	private final IFile file;
	private final IDOMModel model;

	public TranslationValidationReporter(IValidator validator, IReporter reporter, IFile file, IDOMModel model) {
		this.validator = validator;
		this.reporter = reporter;
		this.file = file;
		this.model = model;
	}

	public void clear() {
		if (file == null) {
			return;
		}
		if (reporter != null) {
			reporter.removeAllMessages(validator, file);
		} else {
			// remove by myself?
			String id = TranslationValidator.class.getName();
			try {
				// TaskListHelper API changed
				// TaskListHelper.getTaskList().removeAllTasks(id, this.file,
				// null);
				TaskListHelper.getTaskList().removeAllTasks(file, id, null);
			} catch (CoreException ex) {
				CatValidationPlugin.log(ex);
			}
		}
	}

	public IDOMModel getModel() {
		return model;
	}

	public void addInfo(int targetOffset, int length, String id, String... params) {
		add(IMessage.LOW_SEVERITY, targetOffset, length, id, params);
	}

	public void addWarning(int targetOffset, int length, String id, String... params) {
		add(IMessage.NORMAL_SEVERITY, targetOffset, length, id, params);
	}

	public void addError(int targetOffset, int length, String id, String... params) {
		add(IMessage.HIGH_SEVERITY, targetOffset, length, id, params);
		CatValidationPlugin.log("@@@ Error lineNumber:" + getLineNumbe(model.getStructuredDocument(), targetOffset));
	}

	private void add(int serverity, int targetOffset, int length, String id, String... params) {
		Message message = new MessageForThisClassLoader();
		message.setSeverity(serverity);
		message.setBundleName("message");
		message.setTargetObject(file);
		message.setGroupName("翻訳検証グループ");
		message.setOffset(targetOffset);
		message.setLength(length);
		message.setLineNo(getLineNumbe(model.getStructuredDocument(), targetOffset));
		message.setId(id);
		message.setParams(params);
		reporter.addMessage(validator, message);
	}

	private int getLineNumbe(IDocument iDoc, int targetOffset) {
		try {
			return iDoc.getLineOfOffset(targetOffset) + 1;
		} catch (BadLocationException e) {
			return IMessage.LINENO_UNSET;
		}
	}

	/**
	 * メッセージ・クラスです。
	 * 空継承しているのは、setBundleName で指定したプロパティー・ファイルをロードするときに
	 * メッセージ・クラスから getClass().getClassLoader() し、そのクラス・ローダーから
	 * プラグインのリソースが読み込まれるためです。
	 */
	private static class MessageForThisClassLoader extends Message {
	}
}
