/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.cat.validation.validator;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import jp.sourceforge.mergedoc.pleiades.resource.Property;

import org.eclipse.core.filebuffers.ITextFileBuffer;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExecutableExtension;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.content.IContentDescription;
import org.eclipse.core.runtime.content.IContentType;
import org.eclipse.core.runtime.content.IContentTypeManager;
import org.eclipse.core.runtime.jobs.ISchedulingRule;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IRegion;
import org.eclipse.wst.sse.core.StructuredModelManager;
import org.eclipse.wst.sse.core.internal.FileBufferModelManager;
import org.eclipse.wst.sse.core.internal.provisional.IModelManager;
import org.eclipse.wst.sse.core.internal.provisional.INodeNotifier;
import org.eclipse.wst.sse.core.internal.provisional.IStructuredModel;
import org.eclipse.wst.sse.core.internal.provisional.IndexedRegion;
import org.eclipse.wst.sse.core.internal.provisional.text.IStructuredDocument;
import org.eclipse.wst.sse.core.internal.provisional.text.IStructuredDocumentRegion;
import org.eclipse.wst.sse.core.utils.StringUtils;
import org.eclipse.wst.sse.ui.internal.reconcile.validator.ISourceValidator;
import org.eclipse.wst.validation.AbstractValidator;
import org.eclipse.wst.validation.ValidationEvent;
import org.eclipse.wst.validation.ValidationResult;
import org.eclipse.wst.validation.ValidationState;
import org.eclipse.wst.validation.internal.core.Message;
import org.eclipse.wst.validation.internal.core.ValidationException;
import org.eclipse.wst.validation.internal.operations.IWorkbenchContext;
import org.eclipse.wst.validation.internal.operations.LocalizedMessage;
import org.eclipse.wst.validation.internal.provisional.core.IMessage;
import org.eclipse.wst.validation.internal.provisional.core.IReporter;
import org.eclipse.wst.validation.internal.provisional.core.IValidationContext;
import org.eclipse.wst.validation.internal.provisional.core.IValidatorJob;
import org.eclipse.wst.xml.core.internal.provisional.document.IDOMDocument;
import org.eclipse.wst.xml.core.internal.provisional.document.IDOMElement;
import org.eclipse.wst.xml.core.internal.provisional.document.IDOMModel;
import org.eclipse.wst.xml.core.internal.provisional.document.IDOMText;
import org.w3c.dom.Node;
import org.w3c.dom.Text;

import benten.cat.validation.CatValidationPlugin;
import benten.core.dom.TransUnitDelegate;

/**
 * 翻訳バリデーターです。
 * <p>
 * このクラスは拡張ポイント org.eclipse.wst.validation.validatorV2 の実装です。
 * なお、検証ロジック以外のコードのほとんどは org.eclipse.wst.html.internal.validation.HTMLValidator
 * のコピーです。
 */
@SuppressWarnings("restriction")
public class TranslationValidator extends AbstractValidator implements IValidatorJob, ISourceValidator,
		IExecutableExtension {

	@Override
	public ValidationResult validate(ValidationEvent event, ValidationState state, IProgressMonitor monitor) {
		CatValidationPlugin
				.log("### public ValidationResult validate(ValidationEvent event, ValidationState state, IProgressMonitor monitor) {");
		return null;
	}
	@Override
	public void validationStarting(IProject project, ValidationState state, IProgressMonitor monitor) {
		CatValidationPlugin
				.log("### public void validationStarting(IProject project, ValidationState state, IProgressMonitor monitor) {");
	}
	@Override
	public void validationFinishing(IProject project, ValidationState state, IProgressMonitor monitor) {
		CatValidationPlugin
				.log("### public void validationFinishing(IProject project, ValidationState state, IProgressMonitor monitor) {");
	}

	private static final String XLIFF_CONTENT_TYPE = "benten.core.xliffsource";

	static boolean shouldValidate(IFile file) {
		IResource resource = file;
		do {
			if (resource.isDerived() || resource.isTeamPrivateMember() || !resource.isAccessible()
					|| (resource.getName().charAt(0) == '.' && resource.getType() == IResource.FOLDER)) {
				return false;
			}
			resource = resource.getParent();
		} while ((resource.getType() & IResource.PROJECT) == 0);
		return true;
	}

	private IDocument fDocument;
	private IContentType[] fOtherSupportedContentTypes = null;
	private String[] fAdditionalContentTypesIDs;
	private final IContentType fContentType;

	public TranslationValidator() {
		super();
		IContentTypeManager fContentTypeManager = Platform.getContentTypeManager();
		fContentType = fContentTypeManager.getContentType(XLIFF_CONTENT_TYPE);
	}

	public void cleanup(IReporter reporter) {
	}

	private IContentType[] getOtherSupportedContentTypes() {
		if (fOtherSupportedContentTypes == null) {
			List<IContentType> contentTypes = new ArrayList<IContentType>(3);
			if (fAdditionalContentTypesIDs != null) {
				for (int i = 0; i < fAdditionalContentTypesIDs.length; i++) {
					IContentType type = Platform.getContentTypeManager().getContentType(fAdditionalContentTypesIDs[i]);
					if (type != null) {
						contentTypes.add(type);
					}
				}
			}
			fOtherSupportedContentTypes = contentTypes.toArray(new IContentType[contentTypes.size()]);
		}
		return fOtherSupportedContentTypes;
	}

	protected IDOMModel getModel(IProject project, IFile file) {
		if (project == null || file == null)
			return null;
		if (!file.exists())
			return null;
		if (!canHandle(file))
			return null;

		IStructuredModel model = null;
		IModelManager manager = StructuredModelManager.getModelManager();
		try {
			file.refreshLocal(IResource.DEPTH_ZERO, new NullProgressMonitor());
		} catch (CoreException e) {
			CatValidationPlugin.log(e);
		}
		try {
			try {
				model = manager.getModelForRead(file);
			} catch (UnsupportedEncodingException ex) {
				// retry ignoring META charset for invalid META charset
				// specification
				// recreate input stream, because it is already partially read
				model = manager.getModelForRead(file, new String(), null);
			}
		} catch (UnsupportedEncodingException ex) {
		} catch (IOException ex) {
		} catch (CoreException e) {
			CatValidationPlugin.log(e);
		}

		if (model == null)
			return null;
		if (!(model instanceof IDOMModel)) {
			releaseModel(model);
			return null;
		}
		return (IDOMModel) model;
	}

	protected TranslationValidationReporter getReporter(IReporter reporter, IFile file, IDOMModel model) {
		return new TranslationValidationReporter(this, reporter, file, model);
	}

	private boolean canHandle(IFile file) {
		boolean result = false;
		if (file != null) {
			try {
				IContentDescription contentDescription = file.getContentDescription();
				if (contentDescription != null) {
					IContentType fileContentType = contentDescription.getContentType();
					if (fileContentType.isKindOf(fContentType)) {
						result = true;
					} else {
						IContentType[] otherTypes = getOtherSupportedContentTypes();
						for (int i = 0; i < otherTypes.length; i++) {
							result = result || fileContentType.isKindOf(otherTypes[i]);
						}
					}
				} else if (fContentType != null) {
					result = fContentType.isAssociatedWith(file.getName());
				}
			} catch (CoreException e) {
				// should be rare, but will ignore to avoid logging "encoding
				// exceptions" and the like here.
				// Logger.logException(e);
			}
		}
		return result;
	}

	protected void releaseModel(IStructuredModel model) {
		if (model != null)
			model.releaseFromRead();
	}

	public void validate(IValidationContext helper, IReporter reporter) {
		if (helper == null)
			return;
		if ((reporter != null) && (reporter.isCancelled() == true)) {
			throw new OperationCanceledException();
		}
		String[] deltaArray = helper.getURIs();
		if (deltaArray != null && deltaArray.length > 0) {
			validateDelta(helper, reporter);
		} else {
			validateFull(helper, reporter);
		}
	}

	public void validate(IRegion dirtyRegion, IValidationContext helper, IReporter reporter) {
		if (helper == null || fDocument == null)
			return;

		if ((reporter != null) && (reporter.isCancelled() == true)) {
			throw new OperationCanceledException();
		}

		IStructuredModel model = StructuredModelManager.getModelManager().getExistingModelForRead(fDocument);
		if (model == null)
			return; // error

		try {

			IDOMDocument document = null;
			if (model instanceof IDOMModel) {
				document = ((IDOMModel) model).getDocument();
			}

			if (document == null) {
				model.releaseFromRead();
				return; //ignore
			}

			IPath filePath = null;
			IFile file = null;

			ITextFileBuffer fb = FileBufferModelManager.getInstance().getBuffer(fDocument);
			if (fb != null) {
				filePath = fb.getLocation();

				if (filePath.segmentCount() > 1) {
					file = ResourcesPlugin.getWorkspace().getRoot().getFile(filePath);
					if (!file.isAccessible()) {
						file = null;
					}
				}
			} else {
				filePath = new Path(model.getId());
			}

			// this will be the wrong region if it's Text (instead of Element)
			// we don't know how to validate Text
			IndexedRegion ir = getCoveringNode(dirtyRegion); //  model.getIndexedRegion(dirtyRegion.getOffset());
			if (ir instanceof Text) {
				while (ir != null && ir instanceof Text) {
					// it's assumed that this gets the IndexedRegion to
					// the right of the end offset
					ir = model.getIndexedRegion(ir.getEndOffset());
				}
			}

			if (ir instanceof INodeNotifier) {

				if (reporter != null) {
					TranslationValidationReporter rep = null;
					rep = getReporter(reporter, file, (IDOMModel) model);
					rep.clear();

					Message mess = new LocalizedMessage(IMessage.LOW_SEVERITY, filePath.toString().substring(1));
					reporter.displaySubtask(this, mess);
				}
				//validate(ir);
			}
		} finally {
			if (model != null)
				model.releaseFromRead();
		}
	}

	private IndexedRegion getCoveringNode(IRegion dirtyRegion) {

		IndexedRegion largestRegion = null;
		if (fDocument instanceof IStructuredDocument) {
			IStructuredDocumentRegion[] regions = ((IStructuredDocument) fDocument).getStructuredDocumentRegions(
					dirtyRegion.getOffset(), dirtyRegion.getLength());
			largestRegion = getLargest(regions);
		}
		return largestRegion;
	}

	protected IndexedRegion getLargest(IStructuredDocumentRegion[] sdRegions) {

		if (sdRegions == null || sdRegions.length == 0)
			return null;

		IndexedRegion currentLargest = getCorrespondingNode(sdRegions[0]);
		for (int i = 0; i < sdRegions.length; i++) {
			if (!sdRegions[i].isDeleted()) {
				IndexedRegion corresponding = getCorrespondingNode(sdRegions[i]);

				if (currentLargest instanceof Text)
					currentLargest = corresponding;

				if (corresponding != null) {
					if (!(corresponding instanceof Text)) {
						if (corresponding.getStartOffset() <= currentLargest.getStartOffset()
								&& corresponding.getEndOffset() >= currentLargest.getEndOffset())
							currentLargest = corresponding;
					}
				}

			}
		}
		return currentLargest;
	}

	protected IndexedRegion getCorrespondingNode(IStructuredDocumentRegion sdRegion) {
		IStructuredModel sModel = StructuredModelManager.getModelManager().getExistingModelForRead(fDocument);
		IndexedRegion indexedRegion = null;
		try {
			if (sModel != null)
				indexedRegion = sModel.getIndexedRegion(sdRegion.getStart());
		} finally {
			if (sModel != null)
				sModel.releaseFromRead();
		}
		return indexedRegion;
	}

	public void connect(IDocument document) {
		fDocument = document;
	}

	public void disconnect(IDocument document) {
		fDocument = null;
	}

	private void validate(IReporter reporter, IFile file, IDOMModel model) {
		if (file == null || model == null)
			return; // error
		IDOMDocument document = model.getDocument();
		if (document == null)
			return; // error
		TranslationValidationReporter rep = new TranslationValidationReporter(this, reporter, file, model);
		rep.clear();

		for (TransUnitDelegate unit : TransUnitDelegate.listOf(document)) {
			validateTransUnit(unit, rep);
		}
		//TODO 問題ビュー、リフレッシュ
		//ProblemsView problemsView = ValidationPlugin.getView(ProblemsView.class);
	}

	private void validateContainer(IValidationContext helper, IReporter reporter, IContainer container) {
		try {
			IResource[] resourceArray = container.members(false);
			for (int i = 0; i < resourceArray.length; i++) {
				IResource resource = resourceArray[i];
				if (resource == null || reporter.isCancelled())
					continue;
				if (resource instanceof IFile) {
					Message message = new LocalizedMessage(IMessage.LOW_SEVERITY, resource.getFullPath().toString()
							.substring(1));
					reporter.displaySubtask(this, message);
					validateFile(helper, reporter, (IFile) resource);
				} else if (resource instanceof IContainer) {
					validateContainer(helper, reporter, (IContainer) resource);
				}
			}
		} catch (CoreException ex) {
		}
	}

	private void validateDelta(IValidationContext helper, IReporter reporter) {
		String[] deltaArray = helper.getURIs();
		for (int i = 0; i < deltaArray.length; i++) {
			String delta = deltaArray[i];
			if (delta == null)
				continue;

			if (reporter != null) {
				Message message = new LocalizedMessage(IMessage.LOW_SEVERITY, delta.substring(1));
				reporter.displaySubtask(this, message);
			}

			IResource resource = getResource(delta);
			if (resource == null || !(resource instanceof IFile))
				continue;
			validateFile(helper, reporter, (IFile) resource);
		}
	}

	private void validateFile(IValidationContext helper, IReporter reporter, IFile file) {
		if ((reporter != null) && (reporter.isCancelled() == true)) {
			throw new OperationCanceledException();
		}
		if (!shouldValidate(file)) {
			return;
		}
		IDOMModel model = getModel(file.getProject(), file);
		if (model == null)
			return;

		try {
			validate(reporter, file, model);
		} finally {
			releaseModel(model);
		}
	}

	private void validateFull(IValidationContext helper, IReporter reporter) {
		IProject project = null;
		String[] fileDelta = helper.getURIs();
		if (helper instanceof IWorkbenchContext) {
			IWorkbenchContext wbHelper = (IWorkbenchContext) helper;
			project = wbHelper.getProject();
		} else if (fileDelta.length > 0) {
			// won't work for project validation (b/c nothing in file delta)
			project = getResource(fileDelta[0]).getProject();
		}
		if (project == null)
			return;
		validateContainer(helper, reporter, project);
	}

	public IResource getResource(String delta) {
		return ResourcesPlugin.getWorkspace().getRoot().getFile(new Path(delta));
	}

	public ISchedulingRule getSchedulingRule(IValidationContext helper) {
		return null;
	}

	public IStatus validateInJob(IValidationContext helper, IReporter reporter) throws ValidationException {
		// Exception catching was removed, see
		// https://bugs.eclipse.org/bugs/show_bug.cgi?id=123600
		IStatus status = Status.OK_STATUS;
		validate(helper, reporter);
		return status;
	}

	public void setInitializationData(IConfigurationElement config, String propertyName, Object data)
			throws CoreException {
		fAdditionalContentTypesIDs = new String[0];
		if (data != null) {
			if (data instanceof String && data.toString().length() > 0) {
				fAdditionalContentTypesIDs = StringUtils.unpack(data.toString());
			}
		}
	}

	@Override
	public ValidationResult validate(IResource resource, int kind, ValidationState state, IProgressMonitor monitor) {
		if (resource.getType() != IResource.FILE)
			return null;
		ValidationResult result = new ValidationResult();
		IReporter reporter = result.getReporter(monitor);
		validateFile(null, reporter, (IFile) resource);
		return result;
	}

	//-----------------------------------------------------------------------------
	// ここから XLIFF 固有の検証コード

	private void validateTransUnit(TransUnitDelegate unit, TranslationValidationReporter reporter) {
		if (!unit.hasSource() || !unit.hasTarget()) {
			return;
		}
		String source = unit.getSource();
		String target = unit.getTarget();
		String s = source.trim();
		String t = target.trim();
		int targetOffset = getTargetStartOffset(unit.getTargetNode());

		//---------------------------------------------------------------------
		// 翻訳忘れチェック
		//---------------------------------------------------------------------

		if (!"".equals(s) && "".equals(t)) {
			reporter.addInfo(targetOffset, 0, "NotTranslated");
			// 翻訳忘れの場合、以降のチェックは行わない
			return;
		}
		if (s.equals(t)) {
			reporter.addInfo(targetOffset, 0, "NotTranslated");
			// 翻訳忘れの場合、以降のチェックは行わない
			return;
		}

		// 翻訳が存在する場合、そのテキスト開始オフセットを取得
		int targetTextOffset = getTargetTextStartOffset(unit.getTargetNode()) + target.indexOf(t);

		//---------------------------------------------------------------------
		// 末尾の同一性チェック
		//---------------------------------------------------------------------

		// 末尾の「...」チェック
		if (s.endsWith("...") && !t.endsWith("...")) {
			// 「...」がその他複数を示す場合は「、、、」があれば OK
			if (!t.contains("、、、")) {
				reporter.addError(targetOffset, 0, "ThreeFullStopEllipsisNeeded");
			}
		}
		// 末尾の「..」チェック
		// ただし、以下のチェックでは、翻訳に「...」がある場合はスルーされてしまう。
		else if (s.endsWith("..") && !t.endsWith("..")) {
		}

		// 末尾の「.」チェック
		else if (!s.endsWith("..") && s.endsWith(".") && !t.endsWith("。")) {

			// ・1 単語の場合はチェックしない
			// ・途中に「:」が含まれる場合はチェックしない
			// 例）Incompatible\ file\ format.\ Workspace\ was\ saved\ with\ an\ incompatible\ version\:\ {0}.
			if (s.contains(" ") && !s.contains(":")) {
				reporter.addError(targetOffset, 0, "KutenNeeded");
			}
		}

		// 末尾の「:」チェック
		else if (s.endsWith(":") && !t.endsWith(":")) {
			reporter.addError(targetOffset, 0, "SemiColonNeeded");
		}

		//---------------------------------------------------------------------
		// 英数字隣接空白チェック
		//---------------------------------------------------------------------

		final String IGNORE_CHAR = "\\p{ASCII}（）「」、。…";
		Matcher mat = Pattern.compile("[^" + IGNORE_CHAR + "\\(]\\p{Alnum}").matcher(t);

		while (mat.find()) {
			reporter.addError(targetTextOffset + mat.start(), mat.end() - mat.start(), "test");
		}
		mat = Pattern.compile("\\p{Alnum}[^" + IGNORE_CHAR + "\\)]").matcher(t);
		while (mat.find()) {
			reporter.addError(targetTextOffset + mat.start(), mat.end() - mat.start(), "test");
		}

		//---------------------------------------------------------------------
		// 埋め込み引数の出現数チェック
		//---------------------------------------------------------------------

		// {0} などの埋め込み数チェック
		mat = Pattern.compile("\\{[0-9]\\}").matcher(s);
		while (mat.find()) {
			String group = mat.group();
			if (!t.contains(group)) {
				String[] params = new String[] { mat.group() };
				reporter.addError(targetOffset, 0, "IlligalVariableParameter", params);
			}
		}

		// 改行数チェック
		int eCount = source.length() - source.replace("\n", "").length();
		int jCount = target.length() - target.replace("\n", "").length();
		if (eCount != jCount && eCount < 50) {
			String[] params = new String[] { Integer.toString(eCount), Integer.toString(jCount) };
			reporter.addError(targetOffset, 0, "UnmatchedLineDelimiter", params);
		}

		//---------------------------------------------------------------------
		// 用語チェック
		//---------------------------------------------------------------------

		for (Property term : CatValidationPlugin.vTermProp) {

			String ng = term.key;
			String ok = term.value;

			if (t.contains(ng)) {
				if (ok.equals("")) {
					String[] params = new String[] { ng };
					reporter.addError(targetOffset, 0, "TremValidation", params);
				} else {
					String[] params = new String[] { ng, ok };
					reporter.addError(targetOffset, 0, "TermValidation2", params);
				}
			}
		}

		//---------------------------------------------------------------------
		// 対訳正規表現チェック
		//---------------------------------------------------------------------

		for (Property trans : CatValidationPlugin.vTranslationProp) {

			String enPart = trans.key;
			String jaPart = trans.value;

			if (s.matches(enPart) && !t.matches(jaPart)) {
				String[] params = new String[2];
				params[0] = enPart.replaceAll("(\\(.+?\\)|\\.\\*)", "").replace("\\s", " ");
				params[1] = jaPart.replaceAll("(\\(\\?s\\)|\\.\\*\\?(\\(|)|(|\\))\\.\\*)", "");
				reporter.addError(targetOffset, 0, "TranslationValidation", params);
			}
		}
	}

	private int getTargetTextStartOffset(Node element) {
		IDOMText textNode = (IDOMText) element.getFirstChild();
		return textNode.getStartOffset();
	}

	private int getTargetStartOffset(Node element) {
		return ((IDOMElement) element).getStartOffset();
	}
}
