/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.core.io;

import java.io.IOException;
import java.io.InputStream;
import java.util.LinkedList;
import java.util.List;

public class IniFile {

	private final List<String> list = new LinkedList<String>();

	public IniFile(Object obj, String path) {
		InputStream is = obj.getClass().getClassLoader().getResourceAsStream(path);
		try {
			String s = Files.toString(is, "UTF-8");

			// 改行区切りでリストを作成
			for (String line : s.split("\\s*(\r\n|\n)\\s*")) {

				// 行頭が # の場合は無視
				if (!line.startsWith("#") && line.length() > 0) {
					list.add(line);
				}
			}
		} catch (IOException e) {
			throw new IllegalArgumentException(e);
		} finally {
			Files.closeQuietly(is);
		}
	}

	public List<String> getList() {
		return list;
	}

	public static List<String> asList(Object obj, String path) {
		return new IniFile(obj, path).getList();
	}
}
