/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.core.model;

/**
 * ヘルプ向け XLIFF trans-unit 要素の id 属性を表すクラスです。
 */
public class HelpTransUnitId {

	private static final String PREFIX = "benten://";
	private static final String SEQ_ATTRIBUTE = "?seq=";
	private static final String END_SUFFIX = "END";
	private final String path;
	private final int seq;
	private final String suffix;

	public HelpTransUnitId(String htmlPath, int seq) {
		this(htmlPath, seq, 0, 0);
	}

	public HelpTransUnitId(String htmlPath, int seq, int suffix, int maxSuffix) {
		if (suffix > maxSuffix) {
			throw new IllegalArgumentException("suffix が maxSuffix を超えています。suffix:" + suffix + " maxSuffix:"
					+ maxSuffix);
		}
		this.path = htmlPath.replace('\\', '/').replaceFirst("^/", "");
		this.seq = seq;
		if (suffix == 0) {
			this.suffix = null;
		} else {
			this.suffix = (suffix == maxSuffix) ? END_SUFFIX : String.valueOf(suffix);
		}
	}

	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();
		sb.append(PREFIX);
		sb.append(path);
		sb.append(SEQ_ATTRIBUTE);
		sb.append(seq);
		if (suffix != null && !suffix.equals("")) {
			sb.append("-");
			sb.append(suffix);
		}
		return sb.toString();
	}

	public static boolean hasContinue(String id) {
		if (!id.startsWith(PREFIX)) {
			throw new IllegalArgumentException("id の接頭辞は " + PREFIX + " でなければなりません。");
		}
		return id.contains(SEQ_ATTRIBUTE) && id.matches(".+\\d+-\\d+");
	}
}
