/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.core.model;

import java.io.File;
import java.io.IOException;
import java.util.LinkedList;
import java.util.List;

import benten.core.io.Files;
import blanco.xliff.BlancoXliffParser;
import blanco.xliff.BlancoXliffUtil;
import blanco.xliff.valueobject.BlancoXliff;
import blanco.xliff.valueobject.BlancoXliffFile;
import blanco.xliff.valueobject.BlancoXliffTransUnit;

/**
 * XLIFF モデルとファイルを紐付けて保持するクラスです。
 */
public class Xliff {

	private final static String[] stateItems = {
			// Benten 追加定義
			"", "x-reviewed",
			// XLIFF Version 1.2 の事前定義
			"final", "needs-adaptation", "needs-l10n", "needs-review-adaptation", "needs-review-l10n",
			"needs-review-translation", "needs-translation", "new", "signed-off", "translated" };

	public static String[] getStateItems() {
		String[] newItems = new String[stateItems.length];
		System.arraycopy(stateItems, 0, newItems, 0, stateItems.length);
		return newItems;
	}

	public static Xliff loadInstance(File file) {
		return new Xliff(file, new BlancoXliffParser().parse(file));
	}

	public static Xliff newInstance(File file) {
		return new Xliff(file, new BlancoXliff());
	}

	private final File file;
	private final BlancoXliff blancoXliff;

	private Xliff(File file, BlancoXliff blancoXliff) {
		this.file = file;
		this.blancoXliff = blancoXliff;
	}

	public List<BlancoXliffFile> getFileList() {
		return blancoXliff.getFileList();
	}

	public BlancoXliffFile createBlancoXliffFile() {
		BlancoXliffFile file = new BlancoXliffFile();
		getFileList().add(file);
		return file;
	}

	public List<BlancoXliffTransUnit> getAllTransUnitList() {
		List<BlancoXliffTransUnit> list = new LinkedList<BlancoXliffTransUnit>();
		for (BlancoXliffFile file : getFileList()) {
			list.addAll(file.getBody().getTransUnitList());
		}
		return list;
	}

	public BlancoXliffTransUnit getTransUnit(String id) {
		return BlancoXliffUtil.getTransUnit(blancoXliff, id);
	}

	public void save() throws IOException {
		// trans-unit を持たない file 要素は除外
		// TODO ちなみに、なぜ trans-unit のない file は除外するのでしょうか?
		// → Kashihara: BentenExtractXliffFragmentProcessImpl で XLIFF フラグメントを抽出したときに
		//    抽出条件によっては、空の file 要素ができてしまうためです。
		//    常に空の file 要素は除外でもいいのかなと思ったのですが、この除外処理は汎用的ではないのかもしれません。

		List<BlancoXliffFile> newFileList = new LinkedList<BlancoXliffFile>();
		for (BlancoXliffFile file : getFileList()) {
			if (file.getBody().getTransUnitList().size() > 0) {
				newFileList.add(file);
			}
		}
		if (newFileList.size() > 0) {
			blancoXliff.setFileList(newFileList);
			BlancoXliffUtil.saveXliffInstance(blancoXliff, file.getParentFile(), Files.baseName(file));
		}
	}
}
