/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.cat.core;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import benten.twa.cat.core.valueobject.BentenApplyExistentTranslationProcessInput;
import benten.twa.ui.AbstractTraverseDir;
import blanco.xliff.BlancoXliffParser;
import blanco.xliff.BlancoXliffSerializer;
import blanco.xliff.valueobject.BlancoXliff;
import blanco.xliff.valueobject.BlancoXliffFile;
import blanco.xliff.valueobject.BlancoXliffTransUnit;

public class BentenApplyExistentTranslationProcessImpl extends AbstractTraverseDir implements
		BentenApplyExistentTranslationProcess {
	protected BentenApplyExistentTranslationProcessInput fInput;

	public int execute(final BentenApplyExistentTranslationProcessInput input) throws IOException,
			IllegalArgumentException {
		fInput = input;

		processDir(new File(input.getSourcedir()));

		return 0;
	}

	/**
	 * 処理の中でアイテムが処理されるたびに進捗報告としてコールバックします。
	 *
	 * @param argProgressMessage 現在処理しているアイテムに関するメッセージ。
	 * @return 処理をそのまま継続する場合は false。処理中断をリクエストしたい場合は true。
	 */
	public boolean progress(final String argProgressMessage) {
		return false;
	}

	@Override
	protected boolean canProcess(final File file) {
		return file.getName().toLowerCase().endsWith(".xlf");
	}

	@Override
	protected void processFile(final File fileCurrent, final String baseDir) {
		if (progress("ファイル [" + fileCurrent.getName() + "] を処理中...")) {
			return;
		}

		final File fileTarget = new File(fInput.getTargetdir() + "/" + baseDir + fileCurrent.getName());

		// TODO 同名ファイルのみを処理対象としているが、この考えが妥当かどうかを検証する必要あり。

		if (fileTarget.exists() == false) {
			System.out.println("[" + fileCurrent.getName() + "]の同名ファイルが存在せず。");
			return;
		}
		if (fileTarget.isFile() == false) {
			System.out.println("[" + fileCurrent.getName() + "]はファイルではない。");
			return;
		}

		// TODO 例外処理
		final BlancoXliff xliffFrom = new BlancoXliffParser().parse(fileCurrent);
		final BlancoXliff xliffTo = new BlancoXliffParser().parse(fileTarget);

		final Map<String, BlancoXliffTransUnit> map = new HashMap<String, BlancoXliffTransUnit>(19384);
		for (BlancoXliffFile toFile : xliffTo.getFileList()) {
			for (BlancoXliffTransUnit transUnitTo : toFile.getBody().getTransUnitList()) {
				map.put(transUnitTo.getId(), transUnitTo);
			}
		}

		int transUnitTotalCount = 0;
		for (BlancoXliffFile fromFile : xliffFrom.getFileList()) {
			transUnitTotalCount += fromFile.getBody().getTransUnitList().size();
		}
		beginTask(transUnitTotalCount);

		int transUnitCount = 0;
		for (BlancoXliffFile fromFile : xliffFrom.getFileList()) {
			for (BlancoXliffTransUnit transUnitFrom : fromFile.getBody().getTransUnitList()) {
				if (progress("ファイル [" + fileCurrent.getName() + "] (" + (++transUnitCount) + "/" + transUnitTotalCount
						+ ") を処理中...")) {
					break;
				}

				if (map.get(transUnitFrom.getId()) == null) {
					System.out.println("キー[" + transUnitFrom.getId() + "]が発見できず");
					continue;
				}

				final BlancoXliffTransUnit transUnitTo = map.get(transUnitFrom.getId());

				if (transUnitTo.getTarget() != null && transUnitTo.getTarget().getTarget() != null
						&& transUnitTo.getTarget().getTarget().length() > 0) {
					continue;
				}

				transUnitTo.setTarget(transUnitFrom.getTarget());
			}
		}

		// TODO alt-trans や note をコピーするかどうかの検討が必要。

		if (progress("XLIFF ファイル [" + fileTarget.getName() + "] を作成／更新中...")) {
			return;
		}

		// 書き出し。
		// TODO 例外処理
		new BlancoXliffSerializer().serialize(xliffTo, fileTarget);

		if (progress("XLIFF ファイル [" + fileTarget.getName() + "] を作成／更新しました。")) {
			return;
		}
	}
}
