/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.cat.dialogs;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;

import benten.twa.cat.handler.ApplyExactMatchHandler.ApplyExactMatchStatus;
import benten.twa.ui.dialogs.AbstractWorkflowDialog;
import benten.twa.ui.fields.ResourceFolderTextField;
import benten.ui.UiStatus;
import benten.ui.UiStatusException;

public class ApplyExactMatchDialog extends AbstractWorkflowDialog {

	private final ApplyExactMatchControls controls = new ApplyExactMatchControls();

	public static class ApplyExactMatchControls {
		public ResourceFolderTextField xliffDirText;
		public ResourceFolderTextField tmxDirText;
	}

	public ApplyExactMatchDialog(Shell parentShell, IStructuredSelection selection) {
		super(parentShell, selection, "完全一致訳の適用");
	}

	@Override
	public void createControls(Composite composite) {
		composite.setLayout(new GridLayout(3, false));

		controls.xliffDirText = new ResourceFolderTextField(composite, "XLIFF ディレクトリー");
		controls.xliffDirText.addModifyListener(this);
		if (getSelectionResource() != null) {
			controls.xliffDirText.setText(getSelectionResource().getProject().getName() + "/xliff");
		}

		controls.tmxDirText = new ResourceFolderTextField(composite, "TMX ディレクトリー");
		controls.tmxDirText.addModifyListener(this);
		if (getSelectionResource() != null) {
			controls.tmxDirText.setText(getSelectionResource().getProject().getName() + "/tmx");
		}
	}

	@Override
	public IStatus getResultStatus() {
		ApplyExactMatchStatus status = new ApplyExactMatchStatus();
		status.tmxDir = controls.tmxDirText.getFile();
		status.xliffDir = controls.xliffDirText.getFile();
		status.setProject(controls.xliffDirText.getResource());
		return status;
	}

	@Override
	public IStatus validate() throws UiStatusException {
		validatePath(controls.tmxDirText);
		validatePath(controls.xliffDirText);
		validateFromTo(controls.tmxDirText, controls.xliffDirText);
		return UiStatus.OK_STATUS;
	}
}
