/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.cat.dialogs;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;

import benten.twa.cat.handler.ApplyExistentTranslationHandler.ApplyExistentTranslationStatus;
import benten.twa.ui.dialogs.AbstractWorkflowDialog;
import benten.twa.ui.fields.FolderTextField;
import benten.twa.ui.fields.ResourceFolderTextField;
import benten.ui.UiStatus;
import benten.ui.UiStatusException;

public class ApplyExistentTranslationDialog extends AbstractWorkflowDialog {

	private final ApplyExistentTranslationControls controls = new ApplyExistentTranslationControls();

	public static class ApplyExistentTranslationControls {
		public FolderTextField fromDirText;
		public ResourceFolderTextField toDirText;
	}

	public ApplyExistentTranslationDialog(Shell parentShell, IStructuredSelection selection) {
		super(parentShell, selection, "過去訳の適用");
	}

	@Override
	public void createControls(Composite composite) {
		composite.setLayout(new GridLayout(3, false));

		controls.fromDirText = new FolderTextField(composite, "適用元 XLIFF ディレクトリー");
		controls.fromDirText.addModifyListener(this);
		controls.fromDirText.setText(getSelectionResourceString());

		controls.toDirText = new ResourceFolderTextField(composite, "適用先 XLIFF ディレクトリー");
		controls.toDirText.addModifyListener(this);
		if (getSelectionResource() != null) {
			controls.toDirText.setText(getSelectionResource().getProject().getName() + "/xliff");
		}
	}

	@Override
	public IStatus getResultStatus() {
		ApplyExistentTranslationStatus status = new ApplyExistentTranslationStatus();
		status.fromDir = controls.fromDirText.getFile();
		status.toDir = controls.toDirText.getFile();
		status.setProject(controls.toDirText.getResource());
		return status;
	}

	@Override
	public IStatus validate() throws UiStatusException {
		validatePath(controls.fromDirText);
		validatePath(controls.toDirText);
		//validateFromTo(controls.fromDirText, controls.toDirText);
		return UiStatus.OK_STATUS;
	}
}
