/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.cat.dialogs;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;

import benten.twa.cat.handler.MarkNoTransableHandler.MarkNoTransableStatus;
import benten.twa.ui.dialogs.AbstractWorkflowDialog;
import benten.twa.ui.fields.ResourceFolderTextField;
import benten.ui.UiStatus;
import benten.ui.UiStatusException;

public class MarkNoTransableDialog extends AbstractWorkflowDialog {

	private final MarkNoTransableControls controls = new MarkNoTransableControls();

	public static class MarkNoTransableControls {
		public ResourceFolderTextField targetDirText;
	}

	public MarkNoTransableDialog(Shell parentShell, IStructuredSelection selection) {
		super(parentShell, selection, "非翻訳対象のマーク");
	}

	@Override
	public void createControls(Composite composite) {
		composite.setLayout(new GridLayout(3, false));

		controls.targetDirText = new ResourceFolderTextField(composite, "対象 XLIFF ディレクトリー");
		controls.targetDirText.addModifyListener(this);
		if (getSelectionResource() != null) {
			controls.targetDirText.setText(getSelectionResource().getProject().getName() + "/xliff");
		}
	}

	@Override
	public IStatus getResultStatus() {
		MarkNoTransableStatus status = new MarkNoTransableStatus();
		status.targetDir = controls.targetDirText.getFile();
		status.setProject(controls.targetDirText.getResource());
		return status;
	}

	@Override
	public IStatus validate() throws UiStatusException {
		validatePath(controls.targetDirText);
		return UiStatus.OK_STATUS;
	}
}
