/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.cat.handler;

import java.io.File;
import java.io.IOException;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.dialogs.StatusDialog;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.xml.sax.SAXException;

import benten.twa.cat.core.BentenApplyExactMatchBatchProcess;
import benten.twa.cat.core.BentenApplyExactMatchProcessImpl;
import benten.twa.cat.core.valueobject.BentenApplyExactMatchProcessInput;
import benten.twa.cat.dialogs.ApplyExactMatchDialog;
import benten.twa.ui.jobs.AbstractWorkflowFileHandler;
import benten.ui.UiPlugin;
import benten.ui.UiStatus;
import benten.ui.UiPlugin.Key;

public class ApplyExactMatchHandler extends AbstractWorkflowFileHandler {

	public static class ApplyExactMatchStatus extends UiStatus {
		public File tmxDir;
		public File xliffDir;
	}

	@Override
	protected StatusDialog createDialog(Shell shell, IStructuredSelection selection) {
		setJobMessage("完全一致訳の適用中...");
		return new ApplyExactMatchDialog(shell, selection);
	}

	@Override
	protected void processFile(IStatus iStatus) throws IOException, SAXException {
		final ApplyExactMatchStatus status = (ApplyExactMatchStatus) iStatus;

		final BentenApplyExactMatchProcessInput input = new BentenApplyExactMatchProcessInput();
		input.setTmxdir(status.tmxDir.getAbsolutePath());
		input.setXliffdir(status.xliffDir.getAbsolutePath());

		// ホワイトスペースを無視して TMX を検索するかどうかのフラグ。
		final IPreferenceStore store = UiPlugin.getDefault().getPreferenceStore();
		input.setIgnorewhitespacetmreference(store.getBoolean(Key.IGNORE_WHITESPACE_TM_REFERENCE.name()));

		new BentenApplyExactMatchBatchProcess().validateInput(input);
		new BentenApplyExactMatchProcessImpl() {
			@Override
			protected void beginTask(final int totalWork) {
				monitor.beginTask(null, totalWork);
			}

			@Override
			public boolean progress(String argProgressMessage) {
				monitor.subTask(argProgressMessage);
				monitor.worked(1);
				return monitor.isCanceled();
			}
		}.execute(input);
	}
}
