/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.cat.handler;

import java.io.File;
import java.io.IOException;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.dialogs.StatusDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.xml.sax.SAXException;

import benten.twa.cat.core.BentenApplyExistentTranslationBatchProcess;
import benten.twa.cat.core.BentenApplyExistentTranslationProcessImpl;
import benten.twa.cat.core.valueobject.BentenApplyExistentTranslationProcessInput;
import benten.twa.cat.dialogs.ApplyExistentTranslationDialog;
import benten.twa.ui.jobs.AbstractWorkflowFileHandler;
import benten.ui.UiStatus;

public class ApplyExistentTranslationHandler extends AbstractWorkflowFileHandler {

	public static class ApplyExistentTranslationStatus extends UiStatus {
		public File fromDir;
		public File toDir;
	}

	@Override
	protected StatusDialog createDialog(final Shell shell, final IStructuredSelection selection) {
		setJobMessage("過去訳の適用中...");
		return new ApplyExistentTranslationDialog(shell, selection);
	}

	@Override
	protected void processFile(final IStatus iStatus) throws IOException, SAXException {
		final ApplyExistentTranslationStatus status = (ApplyExistentTranslationStatus) iStatus;

		final BentenApplyExistentTranslationProcessInput input = new BentenApplyExistentTranslationProcessInput();
		input.setSourcedir(status.fromDir.getAbsolutePath());
		input.setTargetdir(status.toDir.getAbsolutePath());

		// TODO 伊賀さん、input に画面の TM 設定を渡したいです。
		// この Input クラスにアクセッサを追加していただけますか？
		// TODO 柏原さん。この場所ではありません。正しくは、ApplyFuzzyMatchHandler です。

		new BentenApplyExistentTranslationBatchProcess().validateInput(input);
		new BentenApplyExistentTranslationProcessImpl() {
			@Override
			protected void beginTask(final int totalWork) {
				monitor.beginTask(null, totalWork);
			}

			@Override
			public boolean progress(String argProgressMessage) {
				monitor.subTask(argProgressMessage);
				monitor.worked(1);
				return monitor.isCanceled();
			}
		}.execute(input);
	}
}
