/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.cat.handler;

import java.io.File;
import java.io.IOException;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.dialogs.StatusDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.xml.sax.SAXException;

import benten.twa.cat.core.BentenMarkNoTransableBatchProcess;
import benten.twa.cat.core.BentenMarkNoTransableProcessImpl;
import benten.twa.cat.core.valueobject.BentenMarkNoTransableProcessInput;
import benten.twa.cat.dialogs.MarkNoTransableDialog;
import benten.twa.ui.jobs.AbstractWorkflowFileHandler;
import benten.ui.UiStatus;

public class MarkNoTransableHandler extends AbstractWorkflowFileHandler {

	public static class MarkNoTransableStatus extends UiStatus {
		public File targetDir;
	}

	@Override
	protected StatusDialog createDialog(final Shell shell, final IStructuredSelection selection) {
		setJobMessage("非翻訳対象のマーク中...");
		return new MarkNoTransableDialog(shell, selection);
	}

	@Override
	protected void processFile(final IStatus iStatus) throws IOException, SAXException {
		final MarkNoTransableStatus status = (MarkNoTransableStatus) iStatus;

		final BentenMarkNoTransableProcessInput input = new BentenMarkNoTransableProcessInput();
		input.setXliffdir(status.targetDir.getAbsolutePath());
		new BentenMarkNoTransableBatchProcess().validateInput(input);
		new BentenMarkNoTransableProcessImpl() {
			@Override
			protected void beginTask(final int totalWork) {
				monitor.beginTask(null, totalWork);
			}

			@Override
			public boolean progress(String argProgressMessage) {
				monitor.subTask(argProgressMessage);
				monitor.worked(1);
				return monitor.isCanceled();
			}
		}.execute(input);
	}
}
