/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.filter.core;

import java.io.File;
import java.io.IOException;

import benten.core.BentenConstants;
import benten.core.io.Files;
import benten.core.io.XliffFileFilter;
import benten.core.model.HelpTransUnitId;
import benten.core.model.Xliff;
import benten.twa.filter.core.valueobject.BentenConvertToXliffProcessInput;
import benten.twa.filter.model.HtmlInlineTextBuilder;
import benten.twa.filter.model.SentenceSplittedList;
import benten.twa.filter.model.HtmlInlineTextBuilder.InlineText;
import benten.twa.ui.AbstractTraverseDir;
import blanco.xliff.BlancoXliffUtil;
import blanco.xliff.valueobject.BlancoXliffBody;
import blanco.xliff.valueobject.BlancoXliffFile;
import blanco.xliff.valueobject.BlancoXliffHeader;
import blanco.xliff.valueobject.BlancoXliffTool;
import blanco.xliff.valueobject.BlancoXliffTransUnit;

public class BentenConvertToXliffProcessImpl extends AbstractTraverseDir implements BentenConvertToXliffProcess {
	protected BentenConvertToXliffProcessInput fInput;

	public void setInput(BentenConvertToXliffProcessInput input) {
		fInput = input;
	}

	public int execute(final BentenConvertToXliffProcessInput input) throws IOException, IllegalArgumentException {
		fInput = input;
		processDir(new File(input.getSourcedir()));

		return 0;
	}

	public boolean progress(String argProgressMessage) {
		return false;
	}

	@Override
	protected boolean canProcess(File file) {
		return file.getName().toLowerCase().endsWith(".html") || file.getName().toLowerCase().endsWith(".htm");
	}

	@Override
	public void processFile(File htmlFile, String baseDir) throws IOException {
		if (progress("ファイル [" + htmlFile.getName() + "] を処理中...")) {
			return;
		}

		File destFile = new File(fInput.getTargetdir(), Files.relativePath(new File(fInput.getSourcedir()), htmlFile));
		File destDir = destFile.getParentFile();

		destDir.mkdirs();
		File outFile = XliffFileFilter.appendExtension(new File(destDir, htmlFile.getName()));

		Xliff xliff = Xliff.newInstance(outFile);
		BlancoXliffFile xliffFile = xliff.createBlancoXliffFile();
		xliffFile.setSourceLanguage("en-US");
		xliffFile.setTargetLanguage("ja-JP");
		xliffFile.setDatatype("html");

		BlancoXliffHeader header = xliffFile.getHeader();
		BlancoXliffBody body = xliffFile.getBody();

		BlancoXliffTool tool = new BlancoXliffTool();
		tool.setToolId(BentenConstants.PROJECT_NAME_LOWER);
		tool.setToolName(BentenConstants.PROJECT_NAME);
		BlancoXliffUtil.setTool(header, tool);

		String path = Files.relativePath(new File(fInput.getSourcedir()).getParentFile(), htmlFile).replace('\\', '/')
				.substring(1);
		int id = 1;

		for (InlineText inline : new HtmlInlineTextBuilder(htmlFile)) {
			String text = inline.getText();
			SentenceSplittedList splittedList = new SentenceSplittedList(text);
			if (splittedList.size() > 1) {
				for (int subId = 0; subId < splittedList.size(); subId++) {
					HelpTransUnitId helpId = new HelpTransUnitId(path, id, subId + 1, splittedList.size());
					createTransUnit(body, helpId, splittedList.get(subId));
				}
			} else {
				HelpTransUnitId helpId = new HelpTransUnitId(path, id);
				createTransUnit(body, helpId, text);
			}
			id++;
		}

		if (progress("XLIFF ファイル [" + outFile.getName() + "] を作成／更新中...")) {
			return;
		}

		xliff.save();

		if (progress("XLIFF ファイル [" + outFile.getName() + "] を作成／更新しました。")) {
			return;
		}
	}

	protected static void createTransUnit(BlancoXliffBody body, HelpTransUnitId helpId, String source) {
		BlancoXliffTransUnit unit = new BlancoXliffTransUnit();
		unit.setId(helpId.toString());
		unit.setSource(source);
		body.getTransUnitList().add(unit);
	}
}
