/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.filter.core;

import java.io.File;
import java.io.IOException;
import java.util.Iterator;

import benten.core.io.Files;
import benten.core.model.HelpTransUnitId;
import benten.core.model.Xliff;
import benten.twa.filter.core.valueobject.BentenExportHtmlProcessInput;
import benten.twa.filter.model.HtmlInlineTextBuilder;
import benten.twa.filter.model.HtmlInlineTextBuilder.InlineText;
import benten.twa.ui.AbstractTraverseDir;
import blanco.xliff.valueobject.BlancoXliffTarget;
import blanco.xliff.valueobject.BlancoXliffTransUnit;

public class BentenExportHtmlProcessImpl extends AbstractTraverseDir implements BentenExportHtmlProcess {
	protected BentenExportHtmlProcessInput fInput;

	public void setInput(BentenExportHtmlProcessInput input) {
		fInput = input;
	}

	public int execute(final BentenExportHtmlProcessInput input) throws IOException, IllegalArgumentException {
		fInput = input;
		processDir(new File(input.getSourcexliffdir()));

		// HTML 以外をコピー
		new AbstractTraverseDir() {
			{
				processDir(new File(fInput.getSourcehtmldir()));
			}

			@Override
			protected void beginTask(final int totalWork) {
			}
			@Override
			protected boolean canProcess(File file) {
				return !(file.getName().toLowerCase().endsWith(".html") || file.getName().toLowerCase()
						.endsWith(".htm"));
			}
			@Override
			protected void processFile(File file, String baseDir) throws IOException {
				String relativePath = Files.relativePath(new File(fInput.getSourcehtmldir()), file);
				File outputFile = new File(fInput.getTargetdir(), relativePath);
				Files.copyFile(file, outputFile);
			}
		};

		return 0;
	}

	public boolean progress(String argProgressMessage) {
		return false;
	}

	@Override
	protected boolean canProcess(File file) {
		return file.getName().toLowerCase().endsWith(".xlf");
	}

	@Override
	public void processFile(File xliffFile, String baseDir) throws IOException {
		if (progress("XLIFF ファイル [" + xliffFile.getName() + "] を処理中...")) {
			return;
		}

		final String relativePath = Files.relativePath(new File(fInput.getSourcexliffdir()), xliffFile);
		final File toDir = new File(fInput.getTargetdir(), relativePath).getParentFile();
		final File htmlDir = new File(fInput.getSourcehtmldir(), relativePath).getParentFile();

		writeHtmlFromXliff(xliffFile, htmlDir, toDir);
	}

	protected void writeHtmlFromXliff(final File xliffFile, final File htmlDir, final File toDir) throws IOException {
		Xliff xliff = Xliff.loadInstance(xliffFile);
		Iterator<BlancoXliffTransUnit> units = xliff.getAllTransUnitList().iterator();
		String htmlFileName = Files.baseName(xliffFile);
		File inHtmlFile = new File(htmlDir, htmlFileName);
		HtmlInlineTextBuilder htmlBuilder = new HtmlInlineTextBuilder(inHtmlFile);

		if (progress("入力 HTML ファイル [" + inHtmlFile.getName() + "] を処理中...")) {
			return;
		}

		for (InlineText inline : htmlBuilder) {
			StringBuilder target = new StringBuilder();

			while (units.hasNext()) {
				BlancoXliffTransUnit unit = units.next();
				BlancoXliffTarget t = unit.getTarget();

				if (t != null && t.getTarget() != null && !t.getTarget().equals("")) {
					target.append(t.getTarget());
				} else {
					target.append(unit.getSource());
				}

				if (!HelpTransUnitId.hasContinue(unit.getId())) {
					break;
				}
			}

			if (target.length() > 0) {
				inline.setText(target.toString());
			} else {
				inline.setText(inline.getText());
			}
		}

		toDir.mkdirs();
		File outHtmlFile = (new File(toDir, htmlFileName));

		if (progress("HTML ファイル [" + outHtmlFile.getName() + "] を作成／更新中...")) {
			return;
		}

		htmlBuilder.writeTo(outHtmlFile, "UTF-8");

		if (progress("HTML ファイル [" + outHtmlFile.getName() + "] を作成／更新しました。")) {
			return;
		}
	}
}
