/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.filter.core;

import java.io.File;
import java.io.IOException;

import org.xml.sax.SAXException;

import benten.core.io.Files;
import benten.twa.filter.core.valueobject.BentenImportHtmlProcessInput;
import benten.twa.ui.AbstractTraverseDir;
import blanco.html.normalizer.util.BlancoHtmlNormalizerUtil;
import blanco.html.parser.util.BlancoHtmlParserUtil;

public class BentenImportHtmlProcessImpl extends AbstractTraverseDir implements BentenImportHtmlProcess {
	protected BentenImportHtmlProcessInput fInput;

	public void setInput(BentenImportHtmlProcessInput input) {
		fInput = input;
	}

	public int execute(final BentenImportHtmlProcessInput input) throws IOException, IllegalArgumentException {
		fInput = input;
		processDir(new File(input.getSourcedir()));

		return 0;
	}

	public boolean progress(String argProgressMessage) {
		return false;
	}

	@Override
	protected boolean canProcess(File file) {
		return file.getName().toLowerCase().endsWith(".html") || file.getName().toLowerCase().endsWith(".htm");
	}

	@Override
	public void processFile(File htmlFile, String baseDir) throws IOException {
		if (progress("ファイル [" + htmlFile.getName() + "] を処理中...")) {
			return;
		}

		byte[] bytes = Files.readFileToByteArray(htmlFile);
		if (fInput.getNormalizehtml()) {
			bytes = BlancoHtmlNormalizerUtil.normalize(bytes);
		}
		String relativePath = Files.relativePath(new File(fInput.getSourcedir()), htmlFile);
		File outputHtmlFile = new File(fInput.getTargetdir(), relativePath);
		outputHtmlFile.getParentFile().mkdirs();
		bytes = resolveContentType(bytes);

		if (progress("HTML ファイル [" + outputHtmlFile.getName() + "] を作成／更新中...")) {
			return;
		}

		Files.writeByteArrayToFile(outputHtmlFile, bytes);

		if (progress("HTML ファイル [" + outputHtmlFile.getName() + "] を作成／更新しました。")) {
			return;
		}
	}

	@Override
	protected void processFilterdFile(final File file, final String baseDir) throws IOException {
		String relativePath = Files.relativePath(new File(fInput.getSourcedir()), file);
		File outputFile = new File(fInput.getTargetdir(), relativePath);
		Files.copyFile(file, outputFile);
	}

	private byte[] resolveContentType(byte[] bytes) throws IOException {
		try {
			String encoding = BlancoHtmlParserUtil.decideEncoding(bytes);
			String html = new String(bytes, encoding);

			if (!html.matches("(?i)(?s)<meta\\s+http-equiv=\"Content-Type\"\\s+content=\"text/html;\\s+charset=.+?\">")) {

				StringBuilder metaTag = new StringBuilder();
				metaTag.append("\n<meta http-equiv=\"Content-Type\" content=\"text/html; charset=");
				metaTag.append(encoding);
				metaTag.append("\">\n");

				String resultHtml = insertMetaTag(html, "<head>", metaTag.toString());
				if (resultHtml == null) {
					resultHtml = insertMetaTag(html, "<html>", metaTag.toString());
				}
				bytes = resultHtml.getBytes(encoding);
			}
		} catch (SAXException e) {
		}
		return bytes;
	}

	private String insertMetaTag(String html, String beforeTag, String metaTag) {
		StringBuilder sb = new StringBuilder();
		int headIndex = html.toLowerCase().indexOf(beforeTag.toLowerCase());
		if (headIndex != -1) {
			sb.append(html.substring(0, headIndex + beforeTag.length()));
			sb.append(metaTag);
			sb.append(html.substring(headIndex + beforeTag.length()));
			return sb.toString();
		}
		return null;
	}
}
