/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.filter.dialogs;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;

import benten.twa.filter.handler.ConvertToXliffHandler.ConvertToXliffStatus;
import benten.twa.ui.dialogs.AbstractWorkflowDialog;
import benten.twa.ui.fields.ResourceFolderTextField;
import benten.ui.UiStatus;
import benten.ui.UiStatusException;

/**
 * HTML－XLIFF 変換ダイアログです。
 */
public class ConvertToXliffDialog extends AbstractWorkflowDialog {

	private final ConvertToXliffControls controls = new ConvertToXliffControls();

	public static class ConvertToXliffControls {
		public ResourceFolderTextField fromDirText;
		public ResourceFolderTextField toDirText;
	}

	public ConvertToXliffDialog(Shell parentShell, IStructuredSelection selection) {
		super(parentShell, selection, "HTML を XLIFF に変換");
	}

	@Override
	public void createControls(Composite composite) {
		composite.setLayout(new GridLayout(3, false));

		controls.fromDirText = new ResourceFolderTextField(composite, "変換元");
		controls.fromDirText.addModifyListener(this);
		if (getSelectionResource() != null) {
			controls.fromDirText.setText(getSelectionResource().getProject().getName() + "/.benten/source");
		}

		controls.toDirText = new ResourceFolderTextField(composite, "変換先");
		controls.toDirText.addModifyListener(this);
		if (getSelectionResource() != null) {
			controls.toDirText.setText(getSelectionResource().getProject().getName() + "/xliff");
		}
	}

	@Override
	public IStatus getResultStatus() {
		ConvertToXliffStatus status = new ConvertToXliffStatus();
		status.fromDir = controls.fromDirText.getFile();
		status.toDir = controls.toDirText.getFile();
		status.setProject(controls.toDirText.getResource());
		return status;
	}

	@Override
	public IStatus validate() throws UiStatusException {
		validatePath(controls.fromDirText);
		validatePath(controls.toDirText);
		// TODO 仮でコメントアウト。
		//validateFromTo(controls.fromDirText, controls.toDirText);
		return UiStatus.OK_STATUS;
	}
}
