/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.filter.handler;

import java.io.File;
import java.io.IOException;

import org.eclipse.core.commands.IHandler;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.dialogs.StatusDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.xml.sax.SAXException;

import benten.twa.filter.core.BentenExportHtmlBatchProcess;
import benten.twa.filter.core.BentenExportHtmlProcessImpl;
import benten.twa.filter.core.valueobject.BentenExportHtmlProcessInput;
import benten.twa.filter.dialogs.ExportHtmlDialog;
import benten.twa.ui.jobs.AbstractWorkflowFileHandler;
import benten.ui.UiStatus;

/**
 * HTML エクスポート・ハンドラーです。
 */
public class ExportHtmlHandler extends AbstractWorkflowFileHandler implements IHandler {

	public static class ExportHtmlStatus extends UiStatus {
		public File fromXliffDir;
		public File fromHtmlDir;
		public File toDir;
	}

	@Override
	protected StatusDialog createDialog(Shell shell, IStructuredSelection selection) {
		setJobMessage("HTML エクスポート中...");
		return new ExportHtmlDialog(shell, selection);
	}

	@Override
	protected void processFile(IStatus iStatus) throws IOException, SAXException {
		final ExportHtmlStatus status = (ExportHtmlStatus) iStatus;

		final BentenExportHtmlProcessInput input = new BentenExportHtmlProcessInput();
		input.setSourcexliffdir(status.fromXliffDir.getAbsolutePath());
		input.setSourcehtmldir(status.fromHtmlDir.getAbsolutePath());
		input.setTargetdir(status.toDir.getAbsolutePath());

		new BentenExportHtmlBatchProcess().validateInput(input);
		new BentenExportHtmlProcessImpl() {
			@Override
			protected void beginTask(final int totalWork) {
				monitor.beginTask(null, totalWork);
			}
			@Override
			public boolean progress(String argProgressMessage) {
				monitor.subTask(argProgressMessage);
				monitor.worked(1);
				return monitor.isCanceled();
			}
		}.execute(input);
	}
}
