/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.tmx.dialogs;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;

import benten.core.io.TmxFileFilter;
import benten.twa.tmx.handler.MergeTmxHandler.MergeTmxStatus;
import benten.twa.ui.dialogs.AbstractWorkflowDialog;
import benten.twa.ui.fields.ResourceFileTextField;
import benten.ui.UiStatus;
import benten.ui.UiStatusException;

public class MergeTmxDialog extends AbstractWorkflowDialog {

	private final MergeTmxControls controls = new MergeTmxControls();

	public static class MergeTmxControls {
		public ResourceFileTextField targetFileText;
		public ResourceFileTextField toFileText;
	}

	public MergeTmxDialog(Shell parentShell, IStructuredSelection selection) {
		super(parentShell, selection, "TMX フラグメントのマージ");
	}

	@Override
	public void createControls(Composite composite) {
		composite.setLayout(new GridLayout(3, false));

		controls.targetFileText = new ResourceFileTextField(composite, "マージ元", TmxFileFilter.DEFAULT_EXTENSION);
		controls.targetFileText.addModifyListener(this);
		if (getSelectionResource() != null) {
			controls.targetFileText.setText(getSelectionResource().getProject().getName() + "/tmx.fragment/");
		}

		controls.toFileText = new ResourceFileTextField(composite, "マージ先", TmxFileFilter.DEFAULT_EXTENSION);
		controls.toFileText.addModifyListener(this);
		if (getSelectionResource() != null) {
			controls.toFileText.setText(getSelectionResource().getProject().getName() + "/tmx/");
		}
	}

	@Override
	public IStatus getResultStatus() {
		MergeTmxStatus status = new MergeTmxStatus();
		status.targetFile = controls.targetFileText.getFile();
		status.toFile = controls.toFileText.getFile();
		return status;
	}

	@Override
	public IStatus validate() throws UiStatusException {
		validatePath(controls.targetFileText);
		validatePath(controls.toFileText);
		validateFromTo(controls.targetFileText, controls.toFileText);
		return UiStatus.OK_STATUS;
	}
}
