/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.tmx.handler;

import java.io.File;
import java.io.IOException;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.dialogs.StatusDialog;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.xml.sax.SAXException;

import benten.twa.tmx.core.BentenExtractTmxFragmentBatchProcess;
import benten.twa.tmx.core.BentenExtractTmxFragmentProcessImpl;
import benten.twa.tmx.core.valueobject.BentenExtractTmxFragmentProcessInput;
import benten.twa.tmx.dialogs.ExtractTmxFragmentDialog;
import benten.twa.ui.jobs.AbstractWorkflowFileHandler;
import benten.ui.UiPlugin;
import benten.ui.UiStatus;
import benten.ui.UiPlugin.Key;

/**
 * TMX フラグメント抽出ハンドラーです。
 */
public class ExtractTmxFragmentHandler extends AbstractWorkflowFileHandler {

	public static class ExtractTmxFragmentStatus extends UiStatus {
		public File targetFile;
		public File toDir;
		public boolean includeDate;
	}

	@Override
	protected StatusDialog createDialog(Shell shell, IStructuredSelection selection) {
		setJobMessage("抽出中...");
		return new ExtractTmxFragmentDialog(shell, selection);
	}

	protected void processFile(IStatus iStatus) throws IOException, SAXException {
		final ExtractTmxFragmentStatus status = (ExtractTmxFragmentStatus) iStatus;

		final BentenExtractTmxFragmentProcessInput input = new BentenExtractTmxFragmentProcessInput();
		input.setXlifffile(status.targetFile.getAbsolutePath());
		input.setTmxdir(status.toDir.getAbsolutePath());
		input.setIncludedate(status.includeDate);

		// ホワイトスペースを無視した内容で TMX を抽出するかどうかのフラグ。
		final IPreferenceStore store = UiPlugin.getDefault().getPreferenceStore();
		input.setIgnorewhitespacetmxextract(store.getBoolean(Key.IGNORE_WHITESPACE_TMX_EXTRACT.name()));

		new BentenExtractTmxFragmentBatchProcess().validateInput(input);
		new BentenExtractTmxFragmentProcessImpl() {
			@Override
			public boolean progress(String argProgressMessage) {
				monitor.subTask(argProgressMessage);
				monitor.worked(1);
				return monitor.isCanceled();
			}
		}.execute(input);
	}
}
