/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.tmx.handler;

import java.io.File;
import java.io.IOException;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.dialogs.StatusDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.xml.sax.SAXException;

import benten.twa.tmx.core.BentenMergeTmxBatchProcess;
import benten.twa.tmx.core.BentenMergeTmxProcessImpl;
import benten.twa.tmx.core.valueobject.BentenMergeTmxProcessInput;
import benten.twa.tmx.dialogs.MergeTmxDialog;
import benten.twa.ui.jobs.AbstractWorkflowFileHandler;
import benten.ui.UiStatus;

public class MergeTmxHandler extends AbstractWorkflowFileHandler {

	public static class MergeTmxStatus extends UiStatus {
		public File targetFile;
		public File toFile;
	}

	@Override
	protected StatusDialog createDialog(Shell shell, IStructuredSelection selection) {
		setJobMessage("マージ中...");
		return new MergeTmxDialog(shell, selection);
	}

	protected void processFile(IStatus iStatus) throws IOException, SAXException {
		final MergeTmxStatus status = (MergeTmxStatus) iStatus;

		final BentenMergeTmxProcessInput input = new BentenMergeTmxProcessInput();
		input.setSourcefile(status.targetFile.getAbsolutePath());
		input.setTargetfile(status.toFile.getAbsolutePath());
		new BentenMergeTmxBatchProcess().validateInput(input);
		new BentenMergeTmxProcessImpl() {
			@Override
			public boolean progress(String argProgressMessage) {
				monitor.subTask(argProgressMessage);
				monitor.worked(1);
				return monitor.isCanceled();
			}
		}.execute(input);
	}
}
