/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.ui.wizards;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExecutableExtension;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.WorkbenchException;
import org.eclipse.ui.dialogs.WizardNewProjectCreationPage;

public class NewProjectCreationWizard extends Wizard implements INewWizard, IExecutableExtension {

	private WizardNewProjectCreationPage page;
	@SuppressWarnings("unused")
	private IConfigurationElement configElement;

	public NewProjectCreationWizard() {
		setWindowTitle("翻訳プロジェクトの作成");
	}

	public void init(IWorkbench workbench, IStructuredSelection selection) {
	}

	public void setInitializationData(IConfigurationElement config, String propertyName, Object data)
			throws CoreException {
		configElement = config;
	}

	@Override
	public void addPages() {
		page = new WizardNewProjectCreationPage("page");
		page.setTitle("翻訳プロジェクトの作成");
		page.setDescription("翻訳プロジェクトを新規作成します。翻訳対象ごとに、翻訳プロジェクトを 1 つ作成して利用してください。");
		addPage(page);
	}

	@Override
	public boolean performFinish() {
		IProject project = page.getProjectHandle();
		try {
			project.create(null);
			project.open(null);

			// インポートされた HTML が格納されるデフォルト・ディレクトリは [/.benten/source] とします。
			// このディレクトリには、現在は html しか格納されませんが、
			// 将来は docx や txt があわせて入ることを想定しています。
			createFolder(project, ".benten");
			createFolder(project, ".benten/source");

			createFolder(project, "xliff");
			createFolder(project, "tmx");
			createFolder(project, "xliff.fragment");
			createFolder(project, "tmx.fragment");

			project.refreshLocal(IResource.DEPTH_INFINITE, null);
			showPerspective();
		} catch (CoreException e) {
			throw new IllegalStateException(e);
		}
		return true;
	}

	private void createFolder(IProject project, String name) throws CoreException {
		project.getFolder(new Path(name)).create(true, true, null);
	}

	private void showPerspective() {
		IWorkbench workbench = PlatformUI.getWorkbench();
		IWorkbenchWindow window = workbench.getActiveWorkbenchWindow();
		IWorkbenchPage page = window.getActivePage();
		IAdaptable input;
		if (page != null)
			input = page.getInput();
		else
			input = ResourcesPlugin.getWorkspace().getRoot();
		try {
			workbench.showPerspective("benten.translationPerspective", window, input);
		} catch (WorkbenchException e) {
			throw new IllegalStateException(e);
		}
	}
}
