/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.xliff.handler;

import java.io.File;
import java.io.IOException;
import java.net.URL;

import junit.framework.TestCase;

import org.eclipse.core.runtime.FileLocator;

import benten.core.io.Files;
import benten.twa.ui.jobs.AbstractWorkflowFileHandler;
import benten.twa.xliff.handler.ExtractXliffFragmentHandler.ExtractXliffFragmentStatus;

/**
 * PDE JUnit テスト。
 */
public class ExtractXliffFragmentHandlerTest extends TestCase {

	private final AbstractWorkflowFileHandler target = new ExtractXliffFragmentHandler();
	private String dataPath;

	private void setDataPath(String caseId) throws IOException {
		String dataDir = "/src/" + getClass().getName().replace('.', '/') + "Data/" + caseId + "/";
		URL dataUrl = getClass().getClassLoader().getResource(dataDir);
		dataPath = FileLocator.resolve(dataUrl).getPath();
	}

	private void assertEqualsFile(String actualFileName, String expectedFileName) throws IOException {
		File actualFile = new File(dataPath + actualFileName);
		File expectedFile = new File(dataPath + expectedFileName);
		assertEquals(Files.readFileToString(expectedFile), Files.readFileToString(actualFile));
	}

	//-------------------------------------------------------------------------

	public void testRun() throws IOException {
		setDataPath("0001");

		ExtractXliffFragmentStatus status = new ExtractXliffFragmentStatus();
		status.fromXliffFile = new File(dataPath + "in/test.html.xlf");
		status.toDir = new File(dataPath + "out");
		status.includeDate = false;
		status.sizeLimit = 10;
		status.state = "x-reviewed";

		target.run(status);

		assertEqualsFile("out/test.html-1.xlf", "out/expected.html-1.xlf");
		assertEqualsFile("out/test.html-2.xlf", "out/expected.html-2.xlf");
	}
}
