/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.xliff.handler;

import java.io.File;
import java.io.IOException;
import java.net.URL;

import junit.framework.TestCase;

import org.eclipse.core.runtime.FileLocator;

import benten.core.io.Files;
import benten.twa.ui.jobs.AbstractWorkflowFileHandler;
import benten.twa.xliff.handler.MergeXliffHandler.MergeXliffStatus;

/**
 * PDE JUnit テスト。
 */
public class MergeXliffHandlerTest extends TestCase {

	private final AbstractWorkflowFileHandler target = new MergeXliffHandler();
	private String dataPath;

	private void setDataPath(String caseId) throws IOException {
		String dataDir = "/src/" + getClass().getName().replace('.', '/') + "Data/" + caseId + "/";
		URL dataUrl = getClass().getClassLoader().getResource(dataDir);
		dataPath = FileLocator.resolve(dataUrl).getPath();
	}

	private void assertEqualsFile(String actualFileName, String expectedFileName) throws IOException {
		File actualFile = new File(dataPath + actualFileName);
		File expectedFile = new File(dataPath + expectedFileName);
		assertEquals(Files.readFileToString(expectedFile), Files.readFileToString(actualFile));
	}

	//-------------------------------------------------------------------------

	public void testRun() throws IOException {
		setDataPath("0001");

		MergeXliffStatus status = new MergeXliffStatus();
		status.fromFile = new File(dataPath + "in/test.html-1.xlf");
		status.toDir = new File(dataPath + "out");

		byte[] original = Files.readFileToByteArray(new File(dataPath + "in/test.html.xlf"));
		Files.writeByteArrayToFile(new File(dataPath + "out/test.html.xlf"), original);

		target.run(status);

		assertEqualsFile("out/test.html.xlf", "in/expected.html.xlf");
	}
}
