/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.xliff.core;

import java.io.File;
import java.io.IOException;

import benten.core.model.Xliff;
import benten.twa.ui.AbstractTraverseDir;
import benten.twa.xliff.core.valueobject.BentenMergeXliffFragmentProcessInput;
import blanco.xliff.valueobject.BlancoXliffTransUnit;

public class BentenMergeXliffFragmentProcessImpl extends AbstractTraverseDir implements BentenMergeXliffFragmentProcess {
	protected BentenMergeXliffFragmentProcessInput fInput;

	public void setInput(BentenMergeXliffFragmentProcessInput input) {
		fInput = input;
	}

	/**
	 * クラスをインスタンス化して処理を実行する際のエントリポイントです。
	 *
	 * @param input 処理の入力パラメータ。
	 * @return 処理の実行結果。
	 * @throws IOException 入出力例外が発生した場合。
	 * @throws IllegalArgumentException 入力値に不正が見つかった場合。
	 */
	public int execute(final BentenMergeXliffFragmentProcessInput input) throws IOException, IllegalArgumentException {
		processDir(new File(input.getTargetxliffdir()));

		return 0;
	}

	/**
	 * 処理の中でアイテムが処理されるたびに進捗報告としてコールバックします。
	 *
	 * @param argProgressMessage 現在処理しているアイテムに関するメッセージ。
	 * @return 処理をそのまま継続する場合は false。処理中断をリクエストしたい場合は true。
	 */
	public boolean progress(final String argProgressMessage) {
		return false;
	}

	@Override
	protected boolean canProcess(final File file) {
		return file.getName().toLowerCase().endsWith(".xlf");
	}

	@Override
	public void processFile(final File toXliffFile, final String baseDir) {
		if (progress("ファイル [" + toXliffFile.getName() + "] を処理中...")) {
			return;
		}

		// TODO マージ機能の強化。

		Xliff inXliff = Xliff.loadInstance(new File(fInput.getSourcexlifffile()));
		Xliff toXliff = Xliff.loadInstance(toXliffFile);

		int transUnitCount = 0;
		int transUnitTotalCount = inXliff.getAllTransUnitList().size();
		for (BlancoXliffTransUnit inUnit : inXliff.getAllTransUnitList()) {
			if (progress("ファイル [" + toXliffFile.getName() + "] (" + (++transUnitCount) + "/" + transUnitTotalCount
					+ ") を処理中...")) {
				break;
			}

			BlancoXliffTransUnit toUnit = toXliff.getTransUnit(inUnit.getId());

			if (toUnit == null) {
				System.out.println("id[" + inUnit.getId() + "] は [" + toXliffFile.getName() + "] に見つからず");
				continue;
			}

			toUnit.setTranslate(inUnit.getTranslate());
			toUnit.setContextGroupList(inUnit.getContextGroupList());
			toUnit.setTarget(inUnit.getTarget());
			toUnit.setNoteList(inUnit.getNoteList());
		}

		if (progress("XLIFF ファイルを作成中...")) {
			return;
		}

		try {
			toXliff.save();
		} catch (IOException e) {
			e.printStackTrace();
			throw new IllegalArgumentException("処理中に例外発生", e);
		}

		if (progress("XLIFF ファイルを作成しました。")) {
			return;
		}
	}
}
