/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.xliff.dialogs;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;

import benten.core.io.XliffFileFilter;
import benten.twa.ui.dialogs.AbstractWorkflowDialog;
import benten.twa.ui.fields.FileTextField;
import benten.twa.ui.fields.ResourceFolderTextField;
import benten.twa.xliff.handler.MergeXliffHandler.MergeXliffStatus;
import benten.ui.UiStatus;
import benten.ui.UiStatusException;

/**
 * XLIFF フラグメント・マージ・ダイアログです。
 */
public class MergeXliffDialog extends AbstractWorkflowDialog {

	private final MergeXliffControls controls = new MergeXliffControls();

	public static class MergeXliffControls {
		public FileTextField fromFileText;
		public ResourceFolderTextField toDirText;
	}

	public MergeXliffDialog(Shell parentShell, IStructuredSelection selection) {
		super(parentShell, selection, "XLIFF フラグメントのマージ");
	}

	@Override
	public void createControls(Composite composite) {
		composite.setLayout(new GridLayout(3, false));

		controls.fromFileText = new FileTextField(composite, "マージ元", XliffFileFilter.DEFAULT_EXTENSION);
		controls.fromFileText.addModifyListener(this);
		controls.fromFileText.setText(getSelectionResourceString());

		controls.toDirText = new ResourceFolderTextField(composite, "マージ先");
		controls.toDirText.addModifyListener(this);
		if (getSelectionResource() != null) {
			controls.toDirText.setText(getSelectionResource().getProject().getName() + "/xliff");
		}
	}

	@Override
	public IStatus getResultStatus() {
		MergeXliffStatus status = new MergeXliffStatus();
		status.fromFile = controls.fromFileText.getFile();
		status.toDir = controls.toDirText.getFile();
		status.setProject(controls.toDirText.getResource());
		return status;
	}

	@Override
	public IStatus validate() throws UiStatusException {
		validatePath(controls.fromFileText);
		validatePath(controls.toDirText);
		//validateFromTo(controls.fromFileText, controls.toDirText);
		return UiStatus.OK_STATUS;
	}
}
