/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.xliff.handler;

import java.io.File;
import java.io.IOException;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.dialogs.StatusDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.xml.sax.SAXException;

import benten.twa.ui.jobs.AbstractWorkflowFileHandler;
import benten.twa.xliff.core.BentenExtractXliffFragmentBatchProcess;
import benten.twa.xliff.core.BentenExtractXliffFragmentProcessImpl;
import benten.twa.xliff.core.valueobject.BentenExtractXliffFragmentProcessInput;
import benten.twa.xliff.dialogs.ExtractXliffFragmentDialog;
import benten.ui.UiStatus;

/**
 * XLIFF フラグメント抽出ハンドラーです。
 */
public class ExtractXliffFragmentHandler extends AbstractWorkflowFileHandler {

	public static class ExtractXliffFragmentStatus extends UiStatus {
		public File fromXliffFile;
		public File toDir;
		public String state;
		public int sizeLimit;
		public boolean includeDate;
	}

	@Override
	protected StatusDialog createDialog(Shell shell, IStructuredSelection selection) {
		setJobMessage("抽出中...");
		return new ExtractXliffFragmentDialog(shell, selection);
	}

	@Override
	protected void processFile(IStatus iStatus) throws IOException, SAXException {
		ExtractXliffFragmentStatus status = (ExtractXliffFragmentStatus) iStatus;

		final BentenExtractXliffFragmentProcessInput input = new BentenExtractXliffFragmentProcessInput();
		input.setSourcexlifffile(status.fromXliffFile.getAbsolutePath());
		input.setTargetxliffdir(status.toDir.getAbsolutePath());
		if (status.state != null && status.state.length() > 0) {
			input.setExtractbystate(status.state);
		}
		if (status.sizeLimit >= 1) {
			input.setLimitsize(status.sizeLimit);
		}
		input.setIncludedate(status.includeDate);
		new BentenExtractXliffFragmentBatchProcess().validateInput(input);
		new BentenExtractXliffFragmentProcessImpl() {
			@Override
			public boolean progress(String argProgressMessage) {
				monitor.subTask(argProgressMessage);
				monitor.worked(1);
				return monitor.isCanceled();
			}
		}.execute(input);
	}
}
