/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.xliff.handler;

import java.io.File;
import java.io.IOException;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.dialogs.StatusDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.xml.sax.SAXException;

import benten.core.io.XliffFileFilter;
import benten.twa.ui.jobs.AbstractWorkflowDirectoryHandler;
import benten.twa.xliff.core.BentenMergeXliffFragmentBatchProcess;
import benten.twa.xliff.core.BentenMergeXliffFragmentProcessImpl;
import benten.twa.xliff.core.valueobject.BentenMergeXliffFragmentProcessInput;
import benten.twa.xliff.dialogs.MergeXliffDialog;
import benten.ui.UiStatus;

/**
 * XLIFF フラグメント・マージ・ハンドラーです。
 */
public class MergeXliffHandler extends AbstractWorkflowDirectoryHandler {

	public static class MergeXliffStatus extends UiStatus {
		public File fromFile;
		public File toDir;
	}

	@Override
	protected StatusDialog createDialog(Shell shell, IStructuredSelection selection) {
		setJobMessage("マージ中...");
		return new MergeXliffDialog(shell, selection);
	}

	@Override
	public void run(IStatus iStatus) {
		MergeXliffStatus status = (MergeXliffStatus) iStatus;
		explore(status, status.toDir, XliffFileFilter.INSTANCE);
	}

	@Override
	protected void processFile(IStatus iStatus, File toXliffFile) throws IOException, SAXException {
		MergeXliffStatus status = (MergeXliffStatus) iStatus;

		final BentenMergeXliffFragmentProcessInput input = new BentenMergeXliffFragmentProcessInput();
		input.setSourcexlifffile(status.fromFile.getAbsolutePath());
		input.setTargetxliffdir(toXliffFile.getAbsolutePath());
		new BentenMergeXliffFragmentBatchProcess().validateInput(input);
		final BentenMergeXliffFragmentProcessImpl impl = new BentenMergeXliffFragmentProcessImpl() {
			@Override
			public boolean progress(final String argProgressMessage) {
				monitor.subTask(argProgressMessage);
				monitor.worked(1);
				return monitor.isCanceled();
			}
		};
		impl.setInput(input);
		impl.processFile(toXliffFile, "");
	}
}
