/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.ui;

import java.io.File;
import java.io.IOException;

/**
 * ディレクトリを再帰的に走査します。
 *
 * TODO 配置場所は core であるべきかも。
 *
 * Ant タスクで利用する場合に こちらのディレクトリ走査クラスを利用することを想定します。
 */
public abstract class AbstractTraverseDir {
	protected final void processDir(final File fileTarget) throws IOException {
		processDir(fileTarget, "");
	}

	private final void processDir(final File fileTarget, final String baseDir) throws IOException {
		if (fileTarget.getName().equals(".svn")) {
			return;
		}
		final File[] files = fileTarget.listFiles();
		if (files == null) {
			return;
		}
		for (File file : files) {
			if (file.isDirectory()) {
				processDir(file, (baseDir.length() == 0 ? "" : baseDir + file.getName() + "/"));
				continue;
			}
			if (file.isFile() == false) {
				continue;
			}
			if (file.canRead() == false) {
				continue;
			}
			if (canProcess(file)) {
				processFile(file, baseDir);
			} else {
				processFilterdFile(file, baseDir);
			}
		}
	}

	/**
	 * タスクの開始時にトータル作業数を報告するためのメソッド。
	 *
	 * @param totalWork
	 */
	protected void beginTask(final int totalWork) {
	}

	/**
	 * このファイルが処理できるものであるかどうかをチェックします。
	 *
	 * 主にファイルの拡張子チェックを実施します。
	 *
	 * @param file
	 * @return
	 */
	protected abstract boolean canProcess(final File file);

	/**
	 * このファイルを処理します。
	 *
	 * @param file 処理対象のファイル。
	 * @param baseDir 外部から呼び出す際には "" を与えます。
	 */
	protected abstract void processFile(final File file, final String baseDir) throws IOException;

	/**
	 * フィルターされた（canProcess() が false）ファイルを処理します。
	 *
	 * @param file フィルターされたファイル。
	 * @param baseDir 外部から呼び出す際には "" を与えます。
	 */
	protected void processFilterdFile(final File file, final String baseDir) throws IOException {
	}
}
