/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.ui.fields;

import java.io.File;

import org.eclipse.core.runtime.Path;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.ui.internal.ide.IDEWorkbenchPlugin;

/**
 * ファイルを選択するためのテキスト・フィールドです。
 */
@SuppressWarnings("restriction")
public class FileTextField extends AbstractBrowseTextField {

	public FileTextField(Composite parent, String labelText, String... extensions) {
		super(parent, labelText, extensions);
	}

	@Override
	protected void openDialog() {
		FileDialog dialog = new FileDialog(text.getShell());
		dialog.setFilterExtensions(extensions);
		dialog.setFilterPath(getDefaultPath());
		String selected = dialog.open();
		if (selected != null) {
			text.setText(selected);
		}
	}

	protected String getDefaultPath() {
		String path = text.getText() == null ? "" : text.getText().trim();
		if (path.length() == 0) {
			String prev = previousInputMap.get(labelText);
			path = (prev == null) ? "" : prev;
		}
		if (path.length() == 0) {
			return IDEWorkbenchPlugin.getPluginWorkspace().getRoot().getLocation().toOSString();
		} else {
			if (new File(path).exists()) {
				path = resolvePath(path);
				return new Path(path).toOSString();
			}
		}
		return "";
	}
}
