/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.ui.fields;

import java.io.File;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.swt.widgets.Composite;

import benten.core.io.ResourceFile;
import benten.twa.ui.dialogs.AbstractResourceSelectionDialog;
import benten.twa.ui.dialogs.ResourceFileSelectionDialog;

/**
 * プロジェクトのファイルを選択するためのテキスト・フィールドです。
 */
public class ResourceFileTextField extends AbstractBrowseTextField {

	public ResourceFileTextField(Composite parent, String labelText, String... extensions) {
		super(parent, labelText, extensions);
	}

	@Override
	protected void openDialog() {
		AbstractResourceSelectionDialog dialog = createDialog();
		String s = text.getText();
		if (s.length() > 0) {
			IResource res = getResource(s);
			if (res != null && res.exists()) {
				dialog.setInitialSelections(new Object[] { res });
			}
		}
		dialog.open();
		Object[] result = dialog.getResult();
		if (result != null && result.length > 0) {
			IResource resource = (IResource) result[0];
			setResource(resource);
		}
	}

	protected AbstractResourceSelectionDialog createDialog() {
		IProject[] projects = ResourcesPlugin.getWorkspace().getRoot().getProjects();
		ResourceFileSelectionDialog dialog = new ResourceFileSelectionDialog(parent.getShell(), projects);
		dialog.setExtensions(extensions);
		return dialog;
	}

	@Override
	public void setText(String path) {
		if (path != null && !path.equals("")) {
			text.setText(path);
		} else {
			String prevDir = previousInputMap.get(labelText);
			if (prevDir != null) {
				text.setText(prevDir == null ? "" : prevDir);
			}
		}
	}

	@Override
	public File getFile() {
		return new File(getResource().getLocation().toOSString());
	}

	public void setResource(IResource resource) {
		text.setText(ResourceFile.relativePath(resource));
	}

	public IResource getResource() {
		return getResource(getText());
	}

	protected IResource getResource(String relativePath) {
		IWorkspaceRoot workspaceRoot = ResourcesPlugin.getWorkspace().getRoot();
		String[] segments = relativePath.split("/");
		String projectName = segments[0];
		IProject project = workspaceRoot.getProject(projectName);
		if (segments.length == 1) {
			return null;
		}
		String path = relativePath.replaceFirst(projectName + "/", "");
		return project.getFile(path);
	}
}
