/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.ui.fields;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.swt.widgets.Composite;

import benten.twa.ui.dialogs.AbstractResourceSelectionDialog;
import benten.twa.ui.dialogs.ResourceFolderSelectionDialog;

/**
 * プロジェクトのフォルダーを選択するためのテキスト・フィールドです。
 */
public class ResourceFolderTextField extends ResourceFileTextField implements IFolderTextField {

	public ResourceFolderTextField(Composite parent, String labelText) {
		super(parent, labelText);
	}

	@Override
	protected AbstractResourceSelectionDialog createDialog() {
		IProject[] projects = ResourcesPlugin.getWorkspace().getRoot().getProjects();
		ResourceFolderSelectionDialog dialog = new ResourceFolderSelectionDialog(parent.getShell(), projects);
		return dialog;
	}

	@Override
	protected IResource getResource(String relativePath) {
		IWorkspaceRoot workspaceRoot = ResourcesPlugin.getWorkspace().getRoot();
		String[] segments = relativePath.split("/");
		String projectName = segments[0];
		IProject project = workspaceRoot.getProject(projectName);
		if (segments.length == 1) {
			return project;
		}
		String path = relativePath.replaceFirst(projectName + "/", "");
		return project.getFolder(path);
	}
}
