/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.ui.jobs;

import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.dialogs.StatusDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.xml.sax.SAXException;

import benten.twa.TwaPlugin;

/**
 * 翻訳ワークフローで使用するディレクトリー・ハンドラー抽象クラスです。
 */
public abstract class AbstractWorkflowDirectoryHandler extends AbstractWorkflowFileHandler {

	@Override
	abstract public void run(IStatus iStatus);

	protected void explore(IStatus status, File folder, FilenameFilter filenameFilter) {
		NullHandler countHandler = new NullHandler();
		countHandler.exploreInternal(status, folder, filenameFilter);
		monitor.beginTask(null, countHandler.getProcessCount());

		exploreInternal(status, folder, filenameFilter);

		if (monitor.isCanceled()) {
			return;
		}
		monitor.done();
	}

	protected void exploreInternal(IStatus status, File folder, FilenameFilter filenameFilter) {
		File[] files = folder.listFiles();
		for (File file : files) {
			if (file.isDirectory()) {

				// このメソッドを再帰呼び出し
				exploreInternal(status, file, filenameFilter);

			} else {
				if (filenameFilter.accept(file, file.getName())) {
					TwaPlugin.log("processFile: " + file);
					try {
						if (monitor.isCanceled()) {
							return;
						}
						monitor.subTask(file.getName());
						monitor.worked(1);

						// ファイルを処理 (テンプレート・メソッド)
						processFile(status, file);

						processCount++;
					} catch (SAXException e) {
						TwaPlugin.log(e);
					} catch (IOException e) {
						TwaPlugin.log(e);
					}
				}
			}
		}
	}

	/**
	 * explore メソッドにより探索されたファイルを処理するメソッドです。
	 * @param iStatus ステータス (処理パラメーター)
	 * @param file 処理対象となるファイル
	 * @throws IOException 入出力例外が発生した場合
	 * @throws SAXException SAX 解析でエラーが発生した場合
	 */
	abstract protected void processFile(IStatus iStatus, File file) throws IOException, SAXException;

	/**
	 * explore メソッドにのフィルターで処理対象外となったファイルを処理するメソッドです。
	 * 必要に応じてオーバーライドします。
	 *
	 * @param iStatus ステータス (処理パラメーター)
	 * @param file 処理対象となるファイル
	 * @throws IOException 入出力例外が発生した場合
	 * @throws SAXException SAX 解析でエラーが発生した場合
	 */
	protected void processFilteredFile(IStatus iStatus, File file) throws IOException, SAXException {
	}

	/*
	 * AbstractWorkflowFileHandler 用のメソッドのため、オーバーライド不可にする。
	 */
	@Override
	final protected void processFile(IStatus iStatus) throws IOException, SAXException {
	}

	/**
	 * 何もしないハンドラー・クラスです。
	 * ただし、カウントは行われます。
	 */
	private static class NullHandler extends AbstractWorkflowDirectoryHandler {
		@Override
		protected void processFile(IStatus status, File file) throws IOException, SAXException {
		}
		@Override
		public void run(IStatus status) {
		}
		@Override
		protected StatusDialog createDialog(Shell shell, IStructuredSelection selection) {
			return null;
		}
	}
}
