/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.ui.jobs;

import java.io.IOException;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.IHandler;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.StatusDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.handlers.HandlerUtil;
import org.xml.sax.SAXException;

import benten.twa.TwaPlugin;
import benten.twa.ui.dialogs.AbstractWorkflowDialog;

/**
 * 翻訳ワークフローで使用するファイル・ハンドラー抽象クラスです。
 * <p>
 * ディレクトリーを階層的に処理する場合は、このクラスのサブクラスである、
 * AbstractWorkflowDirectoryHandler を使用する必要があります。
 */
public abstract class AbstractWorkflowFileHandler extends AbstractHandler implements IHandler {

	protected String jobMessage;
	protected int processCount = 0;
	protected IProgressMonitor monitor = new NullProgressMonitor();

	protected void setJobMessage(String jobMessage) {
		this.jobMessage = jobMessage;
	}

	public Object execute(ExecutionEvent event) throws ExecutionException {
		Shell shell = HandlerUtil.getActiveShell(event);
		ISelection sel = HandlerUtil.getCurrentSelection(event);
		IStructuredSelection selection = StructuredSelection.EMPTY;
		if (sel instanceof IStructuredSelection) {
			selection = (IStructuredSelection) sel;
		}

		// ダイアログを作成 (テンプレート・メソッド)
		AbstractWorkflowDialog dialog = (AbstractWorkflowDialog) createDialog(shell, selection);

		if (dialog != null && dialog.open() == Dialog.OK) {

			// ジョブ開始
			WorkflowJob job = new WorkflowJob(jobMessage);
			job.setStatus(dialog.getStatus());
			job.setHandler(this);
			job.schedule();
		}
		return null;
	}

	abstract protected StatusDialog createDialog(Shell shell, IStructuredSelection selection);

	public void run(IStatus iStatus) {
		try {
			processFile(iStatus);

			if (monitor.isCanceled()) {
				return;
			}
			monitor.done();
			processCount++;
		} catch (IOException e) {
			TwaPlugin.log(e);
		} catch (SAXException e) {
			TwaPlugin.log(e);
		}
	}

	/**
	 * ファイルを処理するメソッドです。
	 * @param iStatus ステータス (処理パラメーター)
	 * @throws IOException 入出力例外が発生した場合
	 * @throws SAXException SAX 解析でエラーが発生した場合
	 */
	abstract protected void processFile(IStatus iStatus) throws IOException, SAXException;

	public int getProcessCount() {
		return processCount;
	}
}
