/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.ui.jobs;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.swt.widgets.Display;

import benten.twa.TwaPlugin;
import benten.ui.UiStatus;

/**
 * 翻訳ワークフロー・ジョブです。
 */
public class WorkflowJob extends Job {

	private IStatus status;
	private AbstractWorkflowFileHandler handler;

	public WorkflowJob(String name) {
		super(name);
		setUser(true);
	}

	public void setStatus(IStatus status) {
		this.status = status;
	}

	public void setHandler(AbstractWorkflowFileHandler handler) {
		this.handler = handler;
	}

	@Override
	protected IStatus run(IProgressMonitor monitor) {
		handler.monitor = monitor;
		handler.run(status);
		if (monitor.isCanceled()) {
			return Status.CANCEL_STATUS;
		}
		if (handler.getProcessCount() == 0) {

			// エラー表示（同期処理。非同期にすると表示してすぐ消えてしまうため。）
			Display.getDefault().syncExec(new Runnable() {
				public void run() {
					UiStatus s = new UiStatus(IStatus.INFO, "処理対象となるファイルがありませんでした。");
					ErrorDialog.openError(null, "Benten 翻訳ワークフロー", null, s);
				}
			});
			return Status.CANCEL_STATUS;
		} else {

			// リフレッシュ (非同期)
			Display.getDefault().asyncExec(new Runnable() {
				public void run() {
					if (status instanceof UiStatus) {
						UiStatus uiStatus = (UiStatus) status;
						if (uiStatus.getProject() != null) {
							try {
								uiStatus.getProject().refreshLocal(IResource.DEPTH_INFINITE, null);
							} catch (CoreException e) {
								TwaPlugin.log(e);
							}
						}
					}
				}
			});

			// 完了メッセージ表示（同期処理。非同期にすると表示してすぐ消えてしまうため。）
			Display.getDefault().syncExec(new Runnable() {
				public void run() {
					showCpmpleteMessage();
				}
			});
			return Status.OK_STATUS;
		}
	}

	protected void showCpmpleteMessage() {
		// ダイアログの場合、完了メッセージなし。
		// ウィザードでは、このメソッドをオーバーライドする。
	}
}
