/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.ui.wizards;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.widgets.Composite;

import benten.twa.ui.dialogs.AbstractWorkflowDialog;


/**
 * ワークフロー・ダイアログをウィザード・ページとして使用するための抽象クラスです。
 */
public abstract class AbstractWorkflowDialogWizardPage extends WizardPage {

	private final IStructuredSelection selection;
	private AbstractWorkflowDialog workflowDialog;

	protected AbstractWorkflowDialogWizardPage(IStructuredSelection selection) {
		super(AbstractWorkflowDialogWizardPage.class.getName());
		this.selection = selection;
	}

	abstract public AbstractWorkflowDialog createWorkflowDialog(Composite parent, IStructuredSelection selection);

	public void createControl(Composite parent) {
		workflowDialog = createWorkflowDialog(parent, selection);
		workflowDialog.setModifyListner(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				IStatus status = workflowDialog.getValidateStatus();
				if (status.isOK()) {
					setErrorMessage(null);
					setPageComplete(true);
				} else {
					setErrorMessage(status.getMessage());
					setPageComplete(false);
				}
			}
		});
		Composite composite = new Composite(parent, SWT.NULL);
		workflowDialog.createControls(composite);
		setControl(composite);
	}

	public boolean hasError() {
		return workflowDialog.hasError();
	}

	public IStatus getStatus() {
		return workflowDialog.getResultStatus();
	}
}
