/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.ui.wizards;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.IExportWizard;
import org.eclipse.ui.IImportWizard;
import org.eclipse.ui.IWorkbench;

/**
 * 一連のウィザード・ページを管理するウィザード抽象クラスです。
 */
public abstract class AbstractWorkflowWizard extends Wizard implements IImportWizard, IExportWizard {

	private IStructuredSelection selection;
	private AbstractWorkflowDialogWizardPage dialogWizardPage;

	@Override
	abstract public void addPages();

	abstract protected void scheduleJob();

	public void init(IWorkbench workbench, IStructuredSelection selection) {
		this.selection = selection;
	}

	@Override
	public void addPage(IWizardPage page) {
		super.addPage(page);
		if (page instanceof AbstractWorkflowDialogWizardPage) {
			// 入力ページ
			dialogWizardPage = (AbstractWorkflowDialogWizardPage) page;
		}
	}

	@Override
	public boolean performFinish() {
		if (dialogWizardPage.hasError()) {
			return false;
		}
		scheduleJob();
		return true;
	}

	protected IStructuredSelection getSelection() {
		return selection;
	}

	protected IStatus getStatus() {
		return dialogWizardPage.getStatus();
	}
}
