/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.ui;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.Status;

/**
 * UI ステータスです。
 */
public class UiStatus extends Status {

	/** OK ステータス */
	public static final UiStatus OK_STATUS = new UiStatus();

	/** プロジェクト */
	private IProject project;

	/**
	 * コンストラクター。
	 */
	public UiStatus() {
		this(OK, null);
	}

	/**
	 * コンストラクター。
	 * @param severity 重大度
	 * @param message メッセージ
	 */
	public UiStatus(int severity, String message) {
		super(severity, UiPlugin.PLUGIN_ID, message);
	}

	/**
	 * コンストラクター。
	 * @param severity 重大度
	 * @param message メッセージ
	 * @param exception 例外
	 */
	public UiStatus(int severity, String message, Throwable exception) {
		super(severity, UiPlugin.PLUGIN_ID, message, exception);
	}

	/**
	 * コンストラクター。
	 * @param severity 重大度
	 * @param code コード
	 * @param message メッセージ
	 * @param exception 例外
	 */
	public UiStatus(int severity, int code, String message, Throwable exception) {
		super(severity, UiPlugin.PLUGIN_ID, code, message, exception);
	}

	/**
	 * プロジェクトを取得します。
	 * @return プロジェクト
	 */
	public IProject getProject() {
		return project;
	}

	/**
	 * プロジェクトを設定します。
	 * @param resource プロジェクトまたはプロジェクトに含まれるリソース
	 */
	public void setProject(IResource resource) {
		if (resource instanceof IProject) {
			this.project = (IProject) resource;
		} else {
			this.project = resource.getProject();
		}
	}
}
