/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.ui;

import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;

/**
 * 基本的なウィジェットを生成する UI ツールキットです。
 */
public class UiToolkit {

	/** 抽象テーブル・ラベル・プロバイダー */
	abstract public static class TableLabelProvider extends LabelProvider implements ITableLabelProvider {
		public Image getColumnImage(Object element, int columnIndex) {
			return null;
		}
	}

	/** 非同期選択変更アダプター */
	abstract public static class AsyncSelectionChangeAdapter implements ISelectionChangedListener {
		final public void selectionChanged(final SelectionChangedEvent event) {
			Display.getCurrent().asyncExec(new Runnable() {
				public void run() {
					IStructuredSelection selection = (IStructuredSelection) event.getSelection();
					selectionChanged(selection);
				}
			});
		}
		abstract public void selectionChanged(IStructuredSelection selection);
	}

	/**
	 * コンストラクター。
	 */
	public UiToolkit() {
	}

	/**
	 * ラベルを生成します。
	 * @param parent 親コンポジット
	 * @param text テキスト
	 * @return ラベル
	 */
	public Label createLabel(Composite parent, String text) {
		Label label = new Label(parent, SWT.NULL);
		label.setText(text);
		return label;
	}

	/**
	 * ラベルを生成します。
	 * @param parent 親コンポジット
	 * @param text テキスト
	 * @param widthHint 横幅ヒント
	 * @return ラベル
	 */
	public Label createLabel(Composite parent, String text, int widthHint) {
		GridData gd = new GridData();
		gd.widthHint = widthHint;
		Label label = createLabel(parent, text);
		label.setLayoutData(gd);
		return label;
	}

	/**
	 * ボタンを生成します。
	 * @param parent 親コンポジット
	 * @param text テキスト
	 * @param widthHint 横幅ヒント
	 * @return ボタン
	 */
	public Button createButton(Composite parent, String text, int widthHint) {
		GridData gd = new GridData();
		gd.widthHint = widthHint;
		Button b = new Button(parent, SWT.PUSH);
		b.setText(text);
		b.setLayoutData(gd);
		return b;
	}

	/**
	 * グリッド・レイアウトを生成します。
	 * @param numColumns 列数
	 * @param margin マージン
	 * @return グリッド・レイアウト
	 */
	public GridLayout createGridLayout(int numColumns, int margin) {
		GridLayout gl = new GridLayout(numColumns, false);
		gl.marginWidth = margin;
		gl.marginHeight = margin;
		return gl;
	}
}
