/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.ui.actions;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWTError;
import org.eclipse.swt.dnd.Clipboard;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;

import benten.ui.UiPlugin;
import benten.ui.views.CopySupportTableView;

public abstract class AbstractCopyAction extends Action {

	private final CopySupportTableView view;
	private Clipboard clipboard;

	public AbstractCopyAction(CopySupportTableView view, String text) {
		super(text);
		this.view = view;
	}

	@Override
	public void run() {
		IStructuredSelection selection = (IStructuredSelection) view.getSelection();
		if (selection.size() == 0) {
			return;
		}
		try {
			if (clipboard == null) {
				clipboard = new Clipboard(view.getSite().getShell().getDisplay());
			}
			clipboard.setContents(new Object[] { asText(selection) }, new Transfer[] { TextTransfer.getInstance() });
		} catch (SWTError e) {
			UiPlugin.log(e);
		}
	}

	abstract protected String asText(IStructuredSelection selection);
}
